from getstanza.tests.fixtures.configurations import *  # noqa: F403, F401
from getstanza.tests.fixtures.grpc import *  # noqa: F403, F401
from getstanza.tests.fixtures.guards import *  # noqa: F403, F401



import pytest
from stanza.hub.v1 import quota_pb2
from stanza.hub.v1.common_pb2 import Local, Quota, Token


def test_guard_allows_default(quota_guard):
    """Check that an unevaluated guard does not block by default."""

    assert quota_guard.guard_config is None
    assert quota_guard.allowed()


@pytest.mark.asyncio
async def test_guard_allows_when_config_null(quota_service, guard_without_config):
    """It should allow if getting the configuration fails (config is null)"""

    await guard_without_config.run()

    quota_service.GetTokenLease.assert_not_called()
    assert guard_without_config.local_status == Local.LOCAL_NOT_EVAL
    assert guard_without_config.token_status == Token.TOKEN_NOT_EVAL
    assert guard_without_config.quota_status == Quota.QUOTA_NOT_EVAL
    assert guard_without_config.error is not None
    assert guard_without_config.allowed()


@pytest.mark.asyncio
async def test_guard_allows_when_config_fetch_error(
    quota_service, guard_without_config_fetch_error
):
    """It should allow if getting the configuration fails (fetch error)"""

    await guard_without_config_fetch_error.run()

    quota_service.GetTokenLease.assert_not_called()
    assert guard_without_config_fetch_error.local_status == Local.LOCAL_NOT_EVAL
    assert guard_without_config_fetch_error.token_status == Token.TOKEN_NOT_EVAL
    assert guard_without_config_fetch_error.quota_status == Quota.QUOTA_NOT_EVAL
    assert guard_without_config_fetch_error.allowed()


@pytest.mark.asyncio
async def test_guard_allows_when_config_fetch_timeout(
    quota_service, guard_without_config_fetch_timeout
):
    """It should allow if getting the configuration fails (fetch timeout)"""

    await guard_without_config_fetch_timeout.run()

    quota_service.GetTokenLease.assert_not_called()
    assert guard_without_config_fetch_timeout.local_status == Local.LOCAL_NOT_EVAL
    assert guard_without_config_fetch_timeout.token_status == Token.TOKEN_NOT_EVAL
    assert guard_without_config_fetch_timeout.quota_status == Quota.QUOTA_NOT_EVAL
    assert guard_without_config_fetch_timeout.allowed()


@pytest.mark.asyncio
async def test_guard_blocks_when_config_not_found(
    quota_service, guard_without_config_not_found
):
    """It should block if getting the configuration fails (not found)"""

    await guard_without_config_not_found.run()

    quota_service.GetTokenLease.assert_not_called()
    assert guard_without_config_not_found.local_status == Local.LOCAL_NOT_EVAL
    assert guard_without_config_not_found.token_status == Token.TOKEN_NOT_EVAL
    assert guard_without_config_not_found.quota_status == Quota.QUOTA_NOT_EVAL
    assert guard_without_config_not_found.blocked()


# ===================
# === Quota Tests ===
# ===================


@pytest.mark.asyncio
async def test_guard_quota_end(quota_guard):
    await quota_guard.run()
    quota_guard.end(0)
    quota_guard.end(quota_guard.success)
    quota_guard.end(quota_guard.failure)


@pytest.mark.asyncio
async def test_guard_quota_allowed(quota_guard, quota_service):
    """Check that quota allowing works in a simple use case."""

    quota_service.GetTokenLease.return_value = quota_pb2.GetTokenLeaseResponse(
        granted=True,
        leases=[
            quota_pb2.TokenLease(
                duration_msec=86400,
                token="00000000",
                feature=None,
                priority_boost=0,
                weight=None,
                reason=None,
                expires_at=None,
                mode=None,
            )
        ],
    )

    await quota_guard.run()
    quota_guard.end(quota_guard.success)

    quota_service.GetTokenLease.assert_called_once()
    assert quota_guard.local_status == Local.LOCAL_NOT_SUPPORTED
    assert quota_guard.token_status == Token.TOKEN_EVAL_DISABLED
    assert quota_guard.quota_status == Quota.QUOTA_GRANTED
    assert quota_guard.quota_token is not None
    assert quota_guard.block_message is None
    assert quota_guard.block_reason is None
    assert quota_guard.allowed()


@pytest.mark.asyncio
async def test_guard_quota_blocked(quota_guard, quota_service):
    """Check that quota blocking works in a simple use case."""

    quota_service.GetTokenLease.return_value = quota_pb2.GetTokenLeaseResponse(
        granted=False,
        leases=[],
    )

    await quota_guard.run()

    quota_service.GetTokenLease.assert_called_once()
    assert quota_guard.local_status == Local.LOCAL_NOT_SUPPORTED
    assert quota_guard.token_status == Token.TOKEN_EVAL_DISABLED
    assert quota_guard.quota_status == Quota.QUOTA_BLOCKED
    assert (
        quota_guard.block_message == "Stanza quota exhausted. Please try again later."
    )
    assert quota_guard.block_reason == "QUOTA_BLOCKED"
    assert quota_guard.blocked()


@pytest.mark.asyncio
async def test_guard_feature_passed(quota_guard_with_feature, quota_service):
    """It should request quota with specified feature."""

    quota_service.GetTokenLease.return_value = quota_pb2.GetTokenLeaseResponse(
        granted=False,
        leases=[],
    )

    await quota_guard_with_feature.run()

    quota_service.GetTokenLease.assert_called_once()

    token_lease_request: quota_pb2.GetTokenLeaseRequest = (
        quota_service.GetTokenLease.call_args.kwargs["request"]
    )
    assert token_lease_request.selector.feature_name == "QuotaGuardFeature"

    assert quota_guard_with_feature.local_status == Local.LOCAL_NOT_SUPPORTED
    assert quota_guard_with_feature.token_status == Token.TOKEN_EVAL_DISABLED
    assert quota_guard_with_feature.quota_status == Quota.QUOTA_BLOCKED
    assert quota_guard_with_feature.blocked()


@pytest.mark.asyncio
async def test_guard_priority_boost_passed(
    quota_guard_with_priority_boost, quota_service
):
    """It should request quota with specified priority boost."""

    quota_service.GetTokenLease.return_value = quota_pb2.GetTokenLeaseResponse(
        granted=False,
        leases=[],
    )

    await quota_guard_with_priority_boost.run()

    quota_service.GetTokenLease.assert_called_once()

    token_lease_request: quota_pb2.GetTokenLeaseRequest = (
        quota_service.GetTokenLease.call_args.kwargs["request"]
    )
    assert token_lease_request.priority_boost == 5

    assert quota_guard_with_priority_boost.local_status == Local.LOCAL_NOT_SUPPORTED
    assert quota_guard_with_priority_boost.token_status == Token.TOKEN_EVAL_DISABLED
    assert quota_guard_with_priority_boost.quota_status == Quota.QUOTA_BLOCKED
    assert quota_guard_with_priority_boost.blocked()


# TODO: Add baggage and context tests


# ===========================
# === Ingress Token Tests ===
# ===========================


@pytest.mark.asyncio
async def test_guard_valid_ingress_token(token_guard, quota_service):
    """It should validate token before proceeding with execution."""

    quota_service.ValidateToken.return_value = quota_pb2.ValidateTokenResponse(
        valid=True,
        tokens_valid=[],
    )

    await token_guard.run(tokens=["valid_token"])

    quota_service.GetTokenLease.assert_not_called()
    quota_service.ValidateToken.assert_called_once()

    assert token_guard.local_status == Local.LOCAL_NOT_SUPPORTED
    assert token_guard.token_status == Token.TOKEN_VALID
    assert token_guard.quota_status == Quota.QUOTA_EVAL_DISABLED
    assert token_guard.allowed()


@pytest.mark.asyncio
async def test_guard_invalid_ingress_token(token_guard, quota_service):
    """It should block invalid tokens before proceeding with execution."""

    quota_service.ValidateToken.return_value = quota_pb2.ValidateTokenResponse(
        valid=False,
        tokens_valid=[],
    )

    await token_guard.run(tokens=["invalid_token"])

    quota_service.GetTokenLease.assert_not_called()
    quota_service.ValidateToken.assert_called_once()

    assert token_guard.local_status == Local.LOCAL_NOT_SUPPORTED
    assert token_guard.token_status == Token.TOKEN_NOT_VALID
    assert token_guard.quota_status == Quota.QUOTA_NOT_EVAL
    assert token_guard.block_message == "Invalid or expired X-Stanza-Token."
    assert token_guard.block_reason == "TOKEN_NOT_VALID"
    assert token_guard.blocked()


@pytest.mark.asyncio
async def test_guard_missing_ingress_token(token_guard, quota_service):
    """It should block missing token before proceeding with execution."""

    await token_guard.run()

    quota_service.GetTokenLease.assert_not_called()
    quota_service.ValidateToken.assert_not_called()

    assert token_guard.local_status == Local.LOCAL_NOT_SUPPORTED
    assert token_guard.token_status == Token.TOKEN_NOT_VALID
    assert token_guard.quota_status == Quota.QUOTA_NOT_EVAL
    assert token_guard.blocked()


# TODO: Add baggage and context tests.


# =========================
# === Report Only Tests ===
# =========================


@pytest.mark.asyncio
async def test_guard_quota_report_only(report_only_guard, quota_service):
    """Check that quota is allowed when report only is enabled."""

    quota_service.GetTokenLease.return_value = quota_pb2.GetTokenLeaseResponse(
        granted=False,
        leases=[],
    )

    await report_only_guard.run()

    quota_service.GetTokenLease.assert_called_once()
    assert report_only_guard.local_status == Local.LOCAL_NOT_SUPPORTED
    assert report_only_guard.token_status == Token.TOKEN_EVAL_DISABLED
    assert report_only_guard.quota_status == Quota.QUOTA_BLOCKED
    assert report_only_guard.allowed()



from google.api import field_behavior_pb2 as _field_behavior_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Health(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    HEALTH_UNSPECIFIED: _ClassVar[Health]
    HEALTH_OK: _ClassVar[Health]
    HEALTH_OVERLOAD: _ClassVar[Health]
    HEALTH_DOWN: _ClassVar[Health]

class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    STATE_UNSPECIFIED: _ClassVar[State]
    STATE_ENABLED: _ClassVar[State]
    STATE_DISABLED: _ClassVar[State]

class Config(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    CONFIG_UNSPECIFIED: _ClassVar[Config]
    CONFIG_CACHED_OK: _ClassVar[Config]
    CONFIG_FETCHED_OK: _ClassVar[Config]
    CONFIG_NOT_FOUND: _ClassVar[Config]
    CONFIG_FETCH_ERROR: _ClassVar[Config]
    CONFIG_FETCH_TIMEOUT: _ClassVar[Config]

class Local(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    LOCAL_UNSPECIFIED: _ClassVar[Local]
    LOCAL_NOT_SUPPORTED: _ClassVar[Local]
    LOCAL_NOT_EVAL: _ClassVar[Local]
    LOCAL_EVAL_DISABLED: _ClassVar[Local]
    LOCAL_ALLOWED: _ClassVar[Local]
    LOCAL_BLOCKED: _ClassVar[Local]
    LOCAL_ERROR: _ClassVar[Local]

class Token(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    TOKEN_UNSPECIFIED: _ClassVar[Token]
    TOKEN_NOT_EVAL: _ClassVar[Token]
    TOKEN_EVAL_DISABLED: _ClassVar[Token]
    TOKEN_NOT_VALID: _ClassVar[Token]
    TOKEN_VALID: _ClassVar[Token]
    TOKEN_VALIDATION_ERROR: _ClassVar[Token]
    TOKEN_VALIDATION_TIMEOUT: _ClassVar[Token]

class Quota(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    QUOTA_UNSPECIFIED: _ClassVar[Quota]
    QUOTA_NOT_EVAL: _ClassVar[Quota]
    QUOTA_EVAL_DISABLED: _ClassVar[Quota]
    QUOTA_LOCAL_ERROR: _ClassVar[Quota]
    QUOTA_BLOCKED: _ClassVar[Quota]
    QUOTA_GRANTED: _ClassVar[Quota]
    QUOTA_ERROR: _ClassVar[Quota]
    QUOTA_TIMEOUT: _ClassVar[Quota]

class Mode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    MODE_UNSPECIFIED: _ClassVar[Mode]
    MODE_NORMAL: _ClassVar[Mode]
    MODE_REPORT_ONLY: _ClassVar[Mode]
HEALTH_UNSPECIFIED: Health
HEALTH_OK: Health
HEALTH_OVERLOAD: Health
HEALTH_DOWN: Health
STATE_UNSPECIFIED: State
STATE_ENABLED: State
STATE_DISABLED: State
CONFIG_UNSPECIFIED: Config
CONFIG_CACHED_OK: Config
CONFIG_FETCHED_OK: Config
CONFIG_NOT_FOUND: Config
CONFIG_FETCH_ERROR: Config
CONFIG_FETCH_TIMEOUT: Config
LOCAL_UNSPECIFIED: Local
LOCAL_NOT_SUPPORTED: Local
LOCAL_NOT_EVAL: Local
LOCAL_EVAL_DISABLED: Local
LOCAL_ALLOWED: Local
LOCAL_BLOCKED: Local
LOCAL_ERROR: Local
TOKEN_UNSPECIFIED: Token
TOKEN_NOT_EVAL: Token
TOKEN_EVAL_DISABLED: Token
TOKEN_NOT_VALID: Token
TOKEN_VALID: Token
TOKEN_VALIDATION_ERROR: Token
TOKEN_VALIDATION_TIMEOUT: Token
QUOTA_UNSPECIFIED: Quota
QUOTA_NOT_EVAL: Quota
QUOTA_EVAL_DISABLED: Quota
QUOTA_LOCAL_ERROR: Quota
QUOTA_BLOCKED: Quota
QUOTA_GRANTED: Quota
QUOTA_ERROR: Quota
QUOTA_TIMEOUT: Quota
MODE_UNSPECIFIED: Mode
MODE_NORMAL: Mode
MODE_REPORT_ONLY: Mode

class GuardSelector(_message.Message):
    __slots__ = ("environment", "name", "tags")
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    environment: str
    name: str
    tags: _containers.RepeatedCompositeFieldContainer[Tag]
    def __init__(self, environment: _Optional[str] = ..., name: _Optional[str] = ..., tags: _Optional[_Iterable[_Union[Tag, _Mapping]]] = ...) -> None: ...

class FeatureSelector(_message.Message):
    __slots__ = ("environment", "names", "tags")
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    NAMES_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    environment: str
    names: _containers.RepeatedScalarFieldContainer[str]
    tags: _containers.RepeatedCompositeFieldContainer[Tag]
    def __init__(self, environment: _Optional[str] = ..., names: _Optional[_Iterable[str]] = ..., tags: _Optional[_Iterable[_Union[Tag, _Mapping]]] = ...) -> None: ...

class GuardFeatureSelector(_message.Message):
    __slots__ = ("environment", "guard_name", "feature_name", "tags")
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    GUARD_NAME_FIELD_NUMBER: _ClassVar[int]
    FEATURE_NAME_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    environment: str
    guard_name: str
    feature_name: str
    tags: _containers.RepeatedCompositeFieldContainer[Tag]
    def __init__(self, environment: _Optional[str] = ..., guard_name: _Optional[str] = ..., feature_name: _Optional[str] = ..., tags: _Optional[_Iterable[_Union[Tag, _Mapping]]] = ...) -> None: ...

class GuardServiceSelector(_message.Message):
    __slots__ = ("environment", "guard_name", "service_name", "service_release", "tags")
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    GUARD_NAME_FIELD_NUMBER: _ClassVar[int]
    SERVICE_NAME_FIELD_NUMBER: _ClassVar[int]
    SERVICE_RELEASE_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    environment: str
    guard_name: str
    service_name: str
    service_release: str
    tags: _containers.RepeatedCompositeFieldContainer[Tag]
    def __init__(self, environment: _Optional[str] = ..., guard_name: _Optional[str] = ..., service_name: _Optional[str] = ..., service_release: _Optional[str] = ..., tags: _Optional[_Iterable[_Union[Tag, _Mapping]]] = ...) -> None: ...

class ServiceSelector(_message.Message):
    __slots__ = ("environment", "name", "release", "tags")
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    NAME_FIELD_NUMBER: _ClassVar[int]
    RELEASE_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    environment: str
    name: str
    release: str
    tags: _containers.RepeatedCompositeFieldContainer[Tag]
    def __init__(self, environment: _Optional[str] = ..., name: _Optional[str] = ..., release: _Optional[str] = ..., tags: _Optional[_Iterable[_Union[Tag, _Mapping]]] = ...) -> None: ...

class HealthByPriority(_message.Message):
    __slots__ = ("priority", "health")
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    HEALTH_FIELD_NUMBER: _ClassVar[int]
    priority: int
    health: Health
    def __init__(self, priority: _Optional[int] = ..., health: _Optional[_Union[Health, str]] = ...) -> None: ...

class Tag(_message.Message):
    __slots__ = ("key", "value")
    KEY_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    key: str
    value: str
    def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...



import asyncio
import inspect
from functools import WRAPPER_ASSIGNMENTS
from typing import Dict, Optional

import getstanza_fastapi.fastapi_guard
from fastapi import Request
from getstanza.client import StanzaClient
from getstanza.configuration import StanzaConfiguration


class StanzaFastAPIClient(StanzaClient):
    """
    SDK client that assists with integrating FastAPI services with Stanza Hub,
    and managing the active service and guard configurations.
    """

    def __init__(self, config: StanzaConfiguration):
        super().__init__(config)

    def stanza_guard(
        self,
        guard_name: str,
        feature_name: Optional[str] = None,
        priority_boost: Optional[int] = None,
        tags: Optional[Dict[str, str]] = None,
    ):
        """Wrap your FastAPI handler with a Stanza guard."""

        def decorator(wrapped):
            wrapped_parameters = inspect.signature(wrapped).parameters.values()

            # We need to make sure that an argument with type 'fastapi.Request'
            # is passed into the handler so that FastAPI gives us request
            # information needed to propagate baggage. If the handler we're
            # wrapping doesn't inject it, then return a wrapper that asks for
            # it.
            wrapper = (
                self.__wrapper_without_request(
                    wrapped,
                    guard_name,
                    feature_name=feature_name,
                    priority_boost=priority_boost,
                    tags=tags,
                )
                if any(p.annotation == Request for p in wrapped_parameters)
                else self.__wrapper_with_request(
                    wrapped,
                    guard_name,
                    feature_name=feature_name,
                    priority_boost=priority_boost,
                    tags=tags,
                )
            )

            # Merge the calling signatures of our decorator with the signature
            # of the handler being passed into it so FastAPI injects the needed
            # dependencies needed by us and also whatever the handler itself
            # needs.
            #
            # The only potential conflicting field is a positional argument
            # with the 'fastapi.Request' type (name and exact position doesn't
            # matter to FastAPI, just the type). That's handled by choosing the
            # wrapper based on whether it's provided by the handler or not.
            wrapper.__signature__ = inspect.Signature(
                parameters=[
                    # Use parameters from the wrapper, save for *args and
                    # **kwargs. These need to come first to ensure any query
                    # parameters with defaults in-use by the handler come after
                    # any positional arguments that we add.
                    *filter(
                        lambda p: p.kind
                        not in (
                            inspect.Parameter.VAR_POSITIONAL,
                            inspect.Parameter.VAR_KEYWORD,
                        ),
                        inspect.signature(wrapper).parameters.values(),
                    ),
                    # Use all parameters from the handler that we're wrapping.
                    *wrapped_parameters,
                ],
                return_annotation=inspect.signature(wrapped).return_annotation,
            )

            # Borrowed from functools, this copies over attributes from the
            # wrapped function into the wrapper such as name and docs so that
            # the wrapper looks like the wrapped function.
            for attr in WRAPPER_ASSIGNMENTS:
                try:
                    value = getattr(wrapped, attr)
                except AttributeError:
                    pass
                else:
                    setattr(wrapper, attr, value)

            return wrapper

        return decorator

    def __wrapper_with_request(
        self,
        wrapped,
        guard_name: str,
        feature_name: Optional[str] = None,
        priority_boost: Optional[int] = None,
        tags: Optional[Dict[str, str]] = None,
    ):
        async def wrapper(request: Request, *args, **kwargs):
            async with getstanza_fastapi.fastapi_guard.StanzaGuard(
                request,
                guard_name,
                feature_name=feature_name,
                priority_boost=priority_boost,
                tags=tags,
            ):
                return (
                    await result
                    if asyncio.iscoroutine(result := wrapped(*args, **kwargs))
                    else result
                )

        return wrapper

    def __wrapper_without_request(
        self,
        wrapped,
        guard_name: str,
        feature_name: Optional[str] = None,
        priority_boost: Optional[int] = None,
        tags: Optional[Dict[str, str]] = None,
    ):
        async def wrapper(*args, **kwargs):
            request = None

            for key, value in kwargs.items():
                if isinstance(value, Request):
                    request = kwargs[key]
                    break

            for arg in args:
                if isinstance(arg, Request):
                    request = arg
                    break

            # This should never happen in practice in runtime. This check exists
            # just to make the type checker happy.
            if request is None:
                raise AttributeError(
                    "Stanza cannot find argument with type 'fastapi.Request' in "
                    "the request handler arguments."
                )

            async with getstanza_fastapi.fastapi_guard.StanzaGuard(
                request,
                guard_name,
                feature_name=feature_name,
                priority_boost=priority_boost,
                tags=tags,
            ):
                return (
                    await result
                    if asyncio.iscoroutine(result := wrapped(*args, **kwargs))
                    else result
                )

        return wrapper



# import os
from typing import Dict, Optional

from opentelemetry.exporter.otlp.proto.grpc.metric_exporter import OTLPMetricExporter
from opentelemetry.metrics import Meter
from opentelemetry.sdk.metrics import MeterProvider
from opentelemetry.sdk.metrics.export import (
    ConsoleMetricExporter,
    MetricExporter,
    PeriodicExportingMetricReader,
)
from opentelemetry.sdk.resources import Resource

# Constants per Stanza SDK spec
METRIC_EXPORT_INTERVAL = 10_000  # 10,000 milliseconds = 10 seconds
METRIC_EXPORT_TIMEOUT = 30_000  # 30,000 milliseconds = 30 seconds


class StanzaMeter:
    def __init__(self, meter: Meter):
        self.AllowedCount = meter.create_counter(
            name="stanza.guard.allowed",
            unit="1",
            description="measures the number of executions that were allowed",
        )
        self.AllowedSuccessCount = meter.create_counter(
            name="stanza.guard.allowed.success",
            unit="1",
            description="measures the number of executions that succeeded",
        )
        self.AllowedFailureCount = meter.create_counter(
            name="stanza.guard.allowed.failure",
            unit="1",
            description="measures the number of executions that failed",
        )
        self.AllowedUnknownCount = meter.create_counter(
            name="stanza.guard.allowed.unknown",
            unit="1",
            description="measures the number of executions where the success (or failure) was unknown",
        )
        self.AllowedDuration = meter.create_histogram(
            name="stanza.guard.allowed.duration",
            unit="ms",
            description="measures the total executions time of guarded requests",
        )
        self.BlockedCount = meter.create_counter(
            name="stanza.guard.blocked",
            unit="1",
            description="measures the number of executions that were backpressured",
        )
        self.FailOpenCount = meter.create_counter(
            name="stanza.guard.failopen",
            unit="1",
            description="measures the number of executions that failed open",
        )


class StanzaMeterProvider:
    @property
    def provider(self):
        return self.__provider

    def __init__(
        self,
        debug: bool,
        insecure: bool,
        resource: Resource,
        endpoint: str,
        headers: Dict[str, str],
    ):
        if debug:
            exporter = self.__debug_meter_provider()
        else:
            exporter = self.__grpc_meter_provider(endpoint, insecure, headers)

        self.__provider = MeterProvider(
            metric_readers=[
                PeriodicExportingMetricReader(
                    exporter=exporter,
                    export_interval_millis=METRIC_EXPORT_INTERVAL,
                    export_timeout_millis=METRIC_EXPORT_TIMEOUT,
                )
            ],
            resource=resource,
        )

    def __debug_meter_provider(self) -> MetricExporter:
        return ConsoleMetricExporter()

    def __grpc_meter_provider(
        self, endpoint: str, insecure: bool, headers: Dict[str, str]
    ) -> MetricExporter:
        return OTLPMetricExporter(
            endpoint=endpoint,
            insecure=insecure,
            # credentials=  # TODO: allow passing credentials
            headers=headers,
            # timeout= # TODO: how is this timeout different from export_timeout_millis?
        )

    def get_meter(
        self, name: str, version: str, schema_url: Optional[str] = None
    ) -> StanzaMeter:
        return StanzaMeter(
            self.__provider.get_meter(name=name, version=version, schema_url=schema_url)
        )



from google.api import annotations_pb2 as _annotations_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2_1
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class GetBearerTokenRequest(_message.Message):
    __slots__ = ("environment",)
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    environment: str
    def __init__(self, environment: _Optional[str] = ...) -> None: ...

class GetBearerTokenResponse(_message.Message):
    __slots__ = ("bearer_token",)
    BEARER_TOKEN_FIELD_NUMBER: _ClassVar[int]
    bearer_token: str
    def __init__(self, bearer_token: _Optional[str] = ...) -> None: ...



import asyncio
import logging
import os
import threading
from collections import defaultdict
from typing import MutableMapping, Optional, TypedDict, cast

import grpc
from getstanza.configuration import StanzaConfiguration
from getstanza.otel import OpenTelemetry
from stanza.hub.v1 import (
    auth_pb2,
    auth_pb2_grpc,
    common_pb2,
    config_pb2,
    config_pb2_grpc,
)

DEFAULT_TIMEOUT = 300

VersionedGuardConfig = TypedDict(
    "VersionedGuardConfig",
    {
        "version": str,
        "config": config_pb2.GuardConfig,
    },
)

service_config_lock = threading.Lock()
guard_config_locks = defaultdict(threading.Lock)


class StanzaHubConfigurationManager:
    """State manager for the active service configuration."""

    @property
    def service_config(self):
        return self.__service_config

    @property
    def otel(self):
        return self.__otel

    def __init__(
        self,
        auth_service: auth_pb2_grpc.AuthServiceStub,
        config_service: config_pb2_grpc.ConfigServiceStub,
        config: StanzaConfiguration,
    ):
        self.config = config

        self.__auth_service = auth_service
        self.__config_service = config_service

        # TODO: Utilize type aliases whenever we upgrade to Python 3.12.
        self.__service_config: Optional[config_pb2.ServiceConfig] = None
        self.__service_config_version: Optional[str] = None
        self.__guard_configs: MutableMapping[str, VersionedGuardConfig] = {}
        self.__otel: Optional[OpenTelemetry] = None

    async def get_guard_config(
        self, guard_name: str
    ) -> tuple[config_pb2.GuardConfig, common_pb2.Config]:
        """Retrieves the guard config for a specified guard."""

        if guard_name in self.__guard_configs:
            return (
                self.__guard_configs[guard_name]["config"],
                common_pb2.Config.CONFIG_CACHED_OK,
            )

        return await self.fetch_guard_config(guard_name)

    async def fetch_otel_bearer_token(self) -> str:
        """Fetch a new bearer token for use with the OTel collector."""

        try:
            bearer_token_response = cast(
                auth_pb2.GetBearerTokenResponse,
                self.__auth_service.GetBearerToken(
                    metadata=self.config.metadata,
                    request=auth_pb2.GetBearerTokenRequest(
                        environment=self.config.environment,
                    ),
                    timeout=DEFAULT_TIMEOUT,
                ),
            )
            return bearer_token_response.bearer_token
        except grpc.RpcError as rpc_error:
            logging.error(str(rpc_error))
            return ""

    async def connect_otel(self):
        """Connect to OTEL collector and create global meter and tracer."""

        if os.environ.get("STANZA_NO_OTEL"):
            # Skip connecting OTEL if STANZA_NO_OTEL environment variable exists.
            return

        if self.__service_config:
            bearer_token = await self.fetch_otel_bearer_token()
            if bearer_token:
                otel = OpenTelemetry(
                    bearer_token=bearer_token,
                    metric_collector_url=self.__service_config.metric_config.collector_url,
                    trace_collector_url=self.__service_config.trace_config.collector_url,
                    trace_sample_rate=self.__service_config.trace_config.sample_rate_default,
                    service_name=str(self.config.service_name or ""),
                    service_release=str(self.config.service_release or ""),
                    environment=str(self.config.environment or ""),
                )
                if otel.new_meter() and otel.new_tracer():
                    self.__otel = otel

    async def fetch_service_config(self):
        """Fetch service configuration changes."""

        with service_config_lock:
            last_version = self.__service_config
            last_version_seen = self.__service_config_version

            try:
                service_config_response = cast(
                    config_pb2.GetServiceConfigResponse,
                    self.__config_service.GetServiceConfig(
                        metadata=self.config.metadata,
                        request=config_pb2.GetServiceConfigRequest(
                            client_id=self.config.client_id,
                            version_seen=last_version_seen,
                            service=common_pb2.ServiceSelector(
                                name=self.config.service_name,
                                release=self.config.service_release,
                                environment=self.config.environment,
                            ),
                        ),
                        timeout=DEFAULT_TIMEOUT,
                    ),
                )
            except grpc.RpcError as rpc_error:
                logging.error(str(rpc_error))
                return

            if service_config_response.config_data_sent:
                self.__service_config = service_config_response.config
                self.__service_config_version = service_config_response.version
                self.config.customer_id = service_config_response.config.customer_id
                logging.debug(
                    "Service config has changed from version '%s' to '%s'",
                    last_version_seen,
                    service_config_response.version,
                )

                if (
                    not self.__otel
                    or not last_version
                    or (
                        last_version.trace_config
                        != service_config_response.config.trace_config
                        or last_version.metric_config
                        != service_config_response.config.metric_config
                    )
                ):
                    await self.connect_otel()

    async def fetch_guard_config(
        self, guard_name: str
    ) -> tuple[config_pb2.GuardConfig, common_pb2.Config]:
        """Fetch guard configuration changes for a specific guard."""

        with guard_config_locks[guard_name]:
            existing_guard_config = self.__guard_configs.get(guard_name)
            last_version_seen = None
            if existing_guard_config:
                last_version_seen = existing_guard_config["version"]

            try:
                guard_config_response = cast(
                    config_pb2.GetGuardConfigResponse,
                    self.__config_service.GetGuardConfig(
                        metadata=self.config.metadata,
                        request=config_pb2.GetGuardConfigRequest(
                            version_seen=last_version_seen,
                            selector=common_pb2.GuardServiceSelector(
                                guard_name=guard_name,
                                service_name=self.config.service_name,
                                service_release=self.config.service_release,
                                environment=self.config.environment,
                            ),
                        ),
                        timeout=DEFAULT_TIMEOUT,
                    ),
                )
            except grpc.RpcError as rpc_error:
                logging.error(str(rpc_error))
                return config_pb2.GuardConfig(), common_pb2.Config.CONFIG_FETCH_ERROR

            if guard_config_response.config_data_sent:
                self.__guard_configs[guard_name] = {
                    "version": guard_config_response.version,
                    "config": guard_config_response.config,
                }
                logging.debug(
                    "Guard config for guard '%s' has changed from version '%s' to '%s'",
                    guard_name,
                    last_version_seen,
                    guard_config_response.version,
                )
                return (
                    self.__guard_configs[guard_name]["config"],
                    common_pb2.Config.CONFIG_FETCHED_OK,
                )
            else:
                return config_pb2.GuardConfig(), common_pb2.Config.CONFIG_FETCHED_OK

    async def refetch_known_guard_configs(self):
        """Refetch all known instantiated guards."""

        tasks = [
            asyncio.create_task(self.fetch_guard_config(guard_name))
            for guard_name in self.__guard_configs
        ]
        if len(tasks) > 0:
            await asyncio.wait(tasks)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from stanza.hub.v1 import health_pb2 as stanza_dot_hub_dot_v1_dot_health__pb2


class HealthServiceStub(object):
    """The Health service definition. This service is used by the Stanza SDK to allow devs to
    make decisions about graceful degradation strategies to apply and to make decisions
    about fail-fast as high up the stack as possible.
    "Keys" - API bearer tokens - are not included in API and should be sent via a X-Stanza-Key header.
    Customer IDs are determined based on the X-Stanza-Key header (each key is associated
    with one and only one customer).
    However, env must always be specified - stanza keys do not have to be
    specific to an environment, so we cannot infer the env from the key.
    Like quota service this should be accessed by SDK via HTTPS.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.QueryGuardHealth = channel.unary_unary(
                '/stanza.hub.v1.HealthService/QueryGuardHealth',
                request_serializer=stanza_dot_hub_dot_v1_dot_health__pb2.QueryGuardHealthRequest.SerializeToString,
                response_deserializer=stanza_dot_hub_dot_v1_dot_health__pb2.QueryGuardHealthResponse.FromString,
                )


class HealthServiceServicer(object):
    """The Health service definition. This service is used by the Stanza SDK to allow devs to
    make decisions about graceful degradation strategies to apply and to make decisions
    about fail-fast as high up the stack as possible.
    "Keys" - API bearer tokens - are not included in API and should be sent via a X-Stanza-Key header.
    Customer IDs are determined based on the X-Stanza-Key header (each key is associated
    with one and only one customer).
    However, env must always be specified - stanza keys do not have to be
    specific to an environment, so we cannot infer the env from the key.
    Like quota service this should be accessed by SDK via HTTPS.
    """

    def QueryGuardHealth(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_HealthServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'QueryGuardHealth': grpc.unary_unary_rpc_method_handler(
                    servicer.QueryGuardHealth,
                    request_deserializer=stanza_dot_hub_dot_v1_dot_health__pb2.QueryGuardHealthRequest.FromString,
                    response_serializer=stanza_dot_hub_dot_v1_dot_health__pb2.QueryGuardHealthResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'stanza.hub.v1.HealthService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class HealthService(object):
    """The Health service definition. This service is used by the Stanza SDK to allow devs to
    make decisions about graceful degradation strategies to apply and to make decisions
    about fail-fast as high up the stack as possible.
    "Keys" - API bearer tokens - are not included in API and should be sent via a X-Stanza-Key header.
    Customer IDs are determined based on the X-Stanza-Key header (each key is associated
    with one and only one customer).
    However, env must always be specified - stanza keys do not have to be
    specific to an environment, so we cannot infer the env from the key.
    Like quota service this should be accessed by SDK via HTTPS.
    """

    @staticmethod
    def QueryGuardHealth(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/stanza.hub.v1.HealthService/QueryGuardHealth',
            stanza_dot_hub_dot_v1_dot_health__pb2.QueryGuardHealthRequest.SerializeToString,
            stanza_dot_hub_dot_v1_dot_health__pb2.QueryGuardHealthResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from stanza.hub.v1 import quota_pb2 as stanza_dot_hub_dot_v1_dot_quota__pb2


class QuotaServiceStub(object):
    """This protocol buffer definition is the source of truth for the Stanza Hub Quota API.
    There is also an OpenAPI spec, generated via https://github.com/grpc-ecosystem/grpc-gateway.

    Quota service is a centralised ratelimiting service, used by Stanza and Browser SDKs to determine whether a given request should be permitted, or whether a Feature should be displayed.

    The Quota service definition. This service is used by the Stanza and Browser SDKs to determine whether quota is available to use services which are subject to centralised ratelimiting.
    "Keys" are not included in API and should be sent via a X-Stanza-Key header.
    Customer IDs are determined based on the X-Stanza-Key header (each key is associated with one and only one customer). However, environment must always be specified when requesting a token - Stanza keys do not have to be specific to an environment, so we cannot infer that from the key alone.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetToken = channel.unary_unary(
                '/stanza.hub.v1.QuotaService/GetToken',
                request_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenRequest.SerializeToString,
                response_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenResponse.FromString,
                )
        self.GetTokenLease = channel.unary_unary(
                '/stanza.hub.v1.QuotaService/GetTokenLease',
                request_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenLeaseRequest.SerializeToString,
                response_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenLeaseResponse.FromString,
                )
        self.SetTokenLeaseConsumed = channel.unary_unary(
                '/stanza.hub.v1.QuotaService/SetTokenLeaseConsumed',
                request_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.SetTokenLeaseConsumedRequest.SerializeToString,
                response_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.SetTokenLeaseConsumedResponse.FromString,
                )
        self.ValidateToken = channel.unary_unary(
                '/stanza.hub.v1.QuotaService/ValidateToken',
                request_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.ValidateTokenRequest.SerializeToString,
                response_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.ValidateTokenResponse.FromString,
                )


class QuotaServiceServicer(object):
    """This protocol buffer definition is the source of truth for the Stanza Hub Quota API.
    There is also an OpenAPI spec, generated via https://github.com/grpc-ecosystem/grpc-gateway.

    Quota service is a centralised ratelimiting service, used by Stanza and Browser SDKs to determine whether a given request should be permitted, or whether a Feature should be displayed.

    The Quota service definition. This service is used by the Stanza and Browser SDKs to determine whether quota is available to use services which are subject to centralised ratelimiting.
    "Keys" are not included in API and should be sent via a X-Stanza-Key header.
    Customer IDs are determined based on the X-Stanza-Key header (each key is associated with one and only one customer). However, environment must always be specified when requesting a token - Stanza keys do not have to be specific to an environment, so we cannot infer that from the key alone.
    """

    def GetToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetTokenLease(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetTokenLeaseConsumed(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ValidateToken(self, request, context):
        """Used by ingress Guards to validate Hub-generated tokens.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QuotaServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetToken': grpc.unary_unary_rpc_method_handler(
                    servicer.GetToken,
                    request_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenRequest.FromString,
                    response_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenResponse.SerializeToString,
            ),
            'GetTokenLease': grpc.unary_unary_rpc_method_handler(
                    servicer.GetTokenLease,
                    request_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenLeaseRequest.FromString,
                    response_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenLeaseResponse.SerializeToString,
            ),
            'SetTokenLeaseConsumed': grpc.unary_unary_rpc_method_handler(
                    servicer.SetTokenLeaseConsumed,
                    request_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.SetTokenLeaseConsumedRequest.FromString,
                    response_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.SetTokenLeaseConsumedResponse.SerializeToString,
            ),
            'ValidateToken': grpc.unary_unary_rpc_method_handler(
                    servicer.ValidateToken,
                    request_deserializer=stanza_dot_hub_dot_v1_dot_quota__pb2.ValidateTokenRequest.FromString,
                    response_serializer=stanza_dot_hub_dot_v1_dot_quota__pb2.ValidateTokenResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'stanza.hub.v1.QuotaService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class QuotaService(object):
    """This protocol buffer definition is the source of truth for the Stanza Hub Quota API.
    There is also an OpenAPI spec, generated via https://github.com/grpc-ecosystem/grpc-gateway.

    Quota service is a centralised ratelimiting service, used by Stanza and Browser SDKs to determine whether a given request should be permitted, or whether a Feature should be displayed.

    The Quota service definition. This service is used by the Stanza and Browser SDKs to determine whether quota is available to use services which are subject to centralised ratelimiting.
    "Keys" are not included in API and should be sent via a X-Stanza-Key header.
    Customer IDs are determined based on the X-Stanza-Key header (each key is associated with one and only one customer). However, environment must always be specified when requesting a token - Stanza keys do not have to be specific to an environment, so we cannot infer that from the key alone.
    """

    @staticmethod
    def GetToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/stanza.hub.v1.QuotaService/GetToken',
            stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenRequest.SerializeToString,
            stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetTokenLease(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/stanza.hub.v1.QuotaService/GetTokenLease',
            stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenLeaseRequest.SerializeToString,
            stanza_dot_hub_dot_v1_dot_quota__pb2.GetTokenLeaseResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetTokenLeaseConsumed(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/stanza.hub.v1.QuotaService/SetTokenLeaseConsumed',
            stanza_dot_hub_dot_v1_dot_quota__pb2.SetTokenLeaseConsumedRequest.SerializeToString,
            stanza_dot_hub_dot_v1_dot_quota__pb2.SetTokenLeaseConsumedResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ValidateToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/stanza.hub.v1.QuotaService/ValidateToken',
            stanza_dot_hub_dot_v1_dot_quota__pb2.ValidateTokenRequest.SerializeToString,
            stanza_dot_hub_dot_v1_dot_quota__pb2.ValidateTokenResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""Extends python-request's Session to integrate with Stanza."""

import asyncio
import logging
from typing import Dict, Optional

from getstanza.client import StanzaClient
from getstanza.guard import Guard
from getstanza.propagation import http_headers_from_context
from requests import PreparedRequest, Request, Response, Session
from requests.utils import check_header_validity


class StanzaSession(Session):
    """
    A subclass of python-requests Session that allows for easily wrapping
    outgoing HTTP requests with Stanza guards.
    """

    def __init__(
        self,
        guard_name: str,
        feature_name: Optional[str] = None,
        priority_boost: Optional[int] = None,
        default_weight: Optional[float] = None,
        tags: Optional[Dict[str, str]] = None,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)

        self.__guard_name = guard_name
        self.__feature_name = feature_name
        self.__priority_boost = priority_boost
        self.__default_weight = default_weight
        self.__tags = tags

        self.__client = StanzaClient.getInstance()

    def prepare_request(self, request: Request) -> PreparedRequest:
        """Constructs a PreparedRequest with additional baggage headers."""

        outgoing_headers = http_headers_from_context()
        for header in outgoing_headers.items():
            check_header_validity(header)

        request.headers = request.headers | outgoing_headers

        return super().prepare_request(request)

    def request(self, *args, **kwargs) -> Response:
        """Calls 'request' along with additional baggage and guard checks."""

        if self.__client.hub is not None:
            # Initialize and run the guard. It's important that initialize on
            # the current thread so that the incoming baggage can be read from.
            guard = Guard(
                self.__client.hub,
                self.__guard_name,
                feature_name=self.__feature_name,
                priority_boost=self.__priority_boost,
                default_weight=self.__default_weight,
                tags=self.__tags,
            )
            asyncio.run_coroutine_threadsafe(
                guard.run(), self.__client.hub.event_loop
            ).result()

            # 🪵 Check for and log any returned error messages
            if guard.error:
                logging.error(guard.error)

            # 🚫 Stanza Guard has *blocked* this workflow log the error and
            # raise an HTTPException with a 429 response code.
            if guard.blocked():
                logging.error(guard.block_message, extra={"reason": guard.block_reason})
                return self.__make_response(guard)

        return super().request(*args, **kwargs)

    def __make_response(self, guard: Guard) -> Response:
        """Craft a fake python-requests response with a 429 error code."""

        def json(*args, **kwargs):
            return {
                "message": guard.block_message,
                "reason": guard.block_reason,
            }

        response = Response()
        response.status_code = 429
        response.headers["Content-Type"] = "application/json"
        response.json = json

        return response



import logging
import re
import threading
from collections import defaultdict
from typing import Any, Callable, Dict, Hashable, Optional

from getstanza.client import StanzaClient
from getstanza.configuration import StanzaConfiguration
from getstanza_sqs.sqs_guard import SQSGuard

QUEUE_ARN_RE = r"^arn:aws:sqs:([a-zA-Z0-9-]+):([0-9]+):(.+)$"

# Maps known (hashable) client objects to Stanza-specific events registered to them. We
# use this to ensure that we don't register event handlers more than once when
# hooking into API calls. We have to keep track of this ourselves since boto3
# doesn't provide a way for us to inspect this at runtime.
_registered_events: defaultdict[Hashable, set[str]] = defaultdict(set)
_registered_events_lock = threading.Lock()


class StanzaSQSClient(StanzaClient):
    """
    SDK client that assists with integrating SQS queue workers with Stanza Hub,
    and managing the active service and guard configurations.
    """

    def __init__(self, config: StanzaConfiguration):
        super().__init__(config)

    def stanza_guard(
        self,
        queue,
        guard_name: str,
        feature_name: Optional[str] = None,
        priority_boost: Optional[int] = None,
        default_weight: Optional[float] = None,
        tags: Optional[Dict[str, str]] = None,
    ):
        """
        Wraps a queue with a Stanza guard. This function will also hook Stanza into
        all 'provide-client-params.sqs.ReceiveMessage' events emitted by the client
        associated with the passed in queue if it's not already registered.
        """

        self.__register_stanza_event_once(
            client=queue.meta.client,
            event_name="provide-client-params.sqs.ReceiveMessage",
            handler=self.__receive_messages,
        )

        if result := re.search(QUEUE_ARN_RE, queue.attributes["QueueArn"]):
            region, account_id, resource_id = result.groups()

            logging.info(
                "Guarding queue with attributes region: %s, account-id: %s, resource-id: %s",
                region,
                account_id,
                resource_id,
            )

        return SQSGuard(
            queue,
            guard_name,
            feature_name=feature_name,
            priority_boost=priority_boost,
            default_weight=default_weight,
            tags=tags,
        )

    def __receive_messages(self, params: dict[str, Any], **kwargs) -> dict[str, Any]:
        """
        Add additional parameters to ReceiveMessage calls that Stanza needs to
        function. Specifically we add "baggage" to the "MessageAttributeNames"
        param so that OTEL baggage can be propagated.
        """

        # TODO: Use approximate stats in queue object to help optimize guard?

        if "MessageAttributeNames" not in params:
            params["MessageAttributeNames"] = []

        if "baggage" not in map(str.casefold, params["MessageAttributeNames"]):
            params["MessageAttributeNames"].append("baggage")

        return params

    def __register_stanza_event_once(self, client, event_name: str, handler: Callable):
        """Registers an event on a client if it's not already registered."""

        with _registered_events_lock:
            if event_name not in _registered_events[client]:
                event_system = client.meta.events
                event_system.register(event_name, handler)
                _registered_events[client].add(event_name)



"""Constants and utility functions that assist with propagation."""

from contextvars import ContextVar
from typing import Mapping, MutableMapping, Optional

from opentelemetry import baggage
from opentelemetry.context import Context as OTELContext
from opentelemetry.propagate import get_global_textmap

STZ_BOOST = "stz-boost"
STZ_FEAT = "stz-feat"

OTEL_CONTEXT_KEY = "stanza-context"
OUTBOUND_HEADERS_KEY = "stanza-outbound-headers"

UBERCTX_STZ_BOOST_KEY = f"uberctx-{STZ_BOOST}"
UBERCTX_STZ_FEAT_KEY = f"uberctx-{STZ_FEAT}"
OT_STZ_BOOST_KEY = f"ot-baggage-{STZ_BOOST}"
OT_STZ_FEAT_KEY = f"ot-baggage-{STZ_FEAT}"

STANZA_INBOUND_HEADERS = [
    UBERCTX_STZ_BOOST_KEY,
    UBERCTX_STZ_FEAT_KEY,
    OT_STZ_BOOST_KEY,
    OT_STZ_FEAT_KEY,
]

# We bind request and baggage to the request context using contextvars.
StanzaContext: ContextVar[OTELContext] = ContextVar(OTEL_CONTEXT_KEY)

# Additional headers for outgoing HTTP requests to support other products.
StanzaOutgoingHeaders: ContextVar[dict[str, str]] = ContextVar(OUTBOUND_HEADERS_KEY)


def context_from_mapping(carrier: Mapping[str, str]) -> OTELContext:
    """Creates a new context from baggage in a map."""

    context = get_global_textmap().extract(carrier)
    StanzaContext.set(context)

    return context


def mapping_from_context(
    carrier: Optional[MutableMapping[str, str]] = None,
) -> MutableMapping[str, str]:
    """Returns a map corresponding to baggage in the current context."""

    _carrier = carrier or {}
    baggage_pairs: dict[str, str] = {}

    for key, value in baggage.get_all(StanzaContext.get()).items():
        baggage_pairs[key] = str(value)

    _carrier["baggage"] = ",".join(
        map(lambda pair: f"{pair[0]}={pair[1]}", baggage_pairs.items())
    )

    return _carrier


def context_from_http_headers(headers: Mapping[str, str]) -> OTELContext:
    """
    Creates a new context with baggage from the baggage header, in addition to
    baggage contained in extra headers used by Jaeger and Datadog.
    """

    def lowercase_pair_key(pair):
        key, value = pair.split("=")

        return f"{key.lower()}={value}"

    otel_baggage = [
        lowercase_pair_key(pair)
        for pair in filter(lambda pair: pair, headers.get("baggage", "").split(","))
    ]

    for inbound_header in STANZA_INBOUND_HEADERS:
        if inbound_header in headers:
            otel_baggage.append(f"{inbound_header}={headers[inbound_header]}")

    carrier = {"baggage": ",".join(otel_baggage)}

    context = get_global_textmap().extract(carrier)
    StanzaContext.set(context)

    return context


def http_headers_from_context(
    headers: Optional[dict[str, str]] = None,
) -> dict[str, str]:
    """Returns HTTP headers corresponding to baggage in the current context."""

    _headers: dict[str, str] = headers or {}
    baggage_pairs: dict[str, str] = {}

    # Add Jaeger and Datadog baggage separately since they should be passed
    # with their own headers rather than with the main baggage header.
    for key, value in StanzaOutgoingHeaders.get().items():
        _headers[key] = value

    # Collect baggage pairs that don't have their own unique headers.
    for key, value in baggage.get_all(StanzaContext.get()).items():
        if key not in STANZA_INBOUND_HEADERS:
            baggage_pairs[key.lower()] = str(value)

    # Add 'baggage' with all pairs that aren't in 'STANZA_INBOUND_HEADERS'.
    _headers["baggage"] = ",".join(
        map(lambda pair: f"{pair[0]}={pair[1]}", baggage_pairs.items())
    )

    return _headers


def get_feature(feature: Optional[str] = None) -> Optional[str]:
    """Return the feature from baggage, or the one provided."""

    _feature: Optional[str] = None
    context = StanzaContext.get()

    if feature is not None:
        _feature = feature
    elif baggage_feat := (
        baggage.get_baggage(STZ_FEAT, context)
        or baggage.get_baggage(UBERCTX_STZ_FEAT_KEY, context)
        or baggage.get_baggage(OT_STZ_FEAT_KEY, context)
    ):
        _feature = str(baggage_feat)

    if not baggage.get_baggage(STZ_FEAT, context) and _feature is not None:
        context = baggage.set_baggage(STZ_FEAT, _feature, context)
        StanzaContext.set(context)

    # Update outgoing headers with additional headers for Jaeger and Datadog.
    if _feature is not None:
        outgoing_headers = StanzaOutgoingHeaders.get({})
        outgoing_headers[UBERCTX_STZ_FEAT_KEY] = _feature
        outgoing_headers[OT_STZ_FEAT_KEY] = _feature
        StanzaOutgoingHeaders.set(outgoing_headers)

    return _feature


def get_priority_boost(priority_boost: Optional[int] = None) -> Optional[int]:
    """Return the priority boost from baggage, or the one provided."""

    _priority_boost: Optional[int] = None
    context = StanzaContext.get()

    if priority_boost is not None:
        _priority_boost = priority_boost

    if baggage_boost := (
        baggage.get_baggage(STZ_BOOST, context)
        or baggage.get_baggage(UBERCTX_STZ_BOOST_KEY, context)
        or baggage.get_baggage(OT_STZ_BOOST_KEY, context)
    ):
        _priority_boost = (
            (_priority_boost or 0) + int(baggage_boost)
            if isinstance(baggage_boost, (str, int))
            else None
        )

    if not baggage.get_baggage(STZ_BOOST, context) and _priority_boost is not None:
        baggage.set_baggage(STZ_BOOST, _priority_boost, context)

    # Update outgoing headers with additional headers for Jaeger and Datadog.
    if _priority_boost is not None:
        outgoing_headers = StanzaOutgoingHeaders.get({})
        outgoing_headers[UBERCTX_STZ_BOOST_KEY] = str(_priority_boost)
        outgoing_headers[OT_STZ_BOOST_KEY] = str(_priority_boost)
        StanzaOutgoingHeaders.set(outgoing_headers)

    return _priority_boost



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stanza/hub/v1/health.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from stanza.hub.v1 import common_pb2 as stanza_dot_hub_dot_v1_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1astanza/hub/v1/health.proto\x12\rstanza.hub.v1\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a\x1astanza/hub/v1/common.proto\"\x99\x01\n\x17QueryGuardHealthRequest\x12?\n\x08selector\x18\x01 \x01(\x0b\x32#.stanza.hub.v1.GuardFeatureSelectorR\x08selector\x12*\n\x0epriority_boost\x18\x04 \x01(\x05H\x00R\rpriorityBoost\x88\x01\x01\x42\x11\n\x0f_priority_boost\"I\n\x18QueryGuardHealthResponse\x12-\n\x06health\x18\x01 \x01(\x0e\x32\x15.stanza.hub.v1.HealthR\x06health2\xc6\x02\n\rHealthService\x12\xb4\x02\n\x10QueryGuardHealth\x12&.stanza.hub.v1.QueryGuardHealthRequest\x1a\'.stanza.hub.v1.QueryGuardHealthResponse\"\xce\x01\x92\x41\xaf\x01\x12\x10Get Guard Health\x1a\x61Used by SDK to allow developers to make decisions about graceful degradation of backend services.J8\n\x03\x32\x30\x30\x12\x31\n\x02OK\x12+\n)\x1a\'.stanza.hub.v1.QueryGuardHealthResponse\x82\xd3\xe4\x93\x02\x15\"\x10/v1/health/guard:\x01*B\x8b\x01\n\x11\x63om.stanza.hub.v1B\x0bHealthProtoP\x01Z\x13stanza/hub/v1;hubpb\xa2\x02\x03SHX\xaa\x02\rStanza.Hub.V1\xca\x02\rStanza\\Hub\\V1\xe2\x02\x19Stanza\\Hub\\V1\\GPBMetadata\xea\x02\x0fStanza::Hub::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stanza.hub.v1.health_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\021com.stanza.hub.v1B\013HealthProtoP\001Z\023stanza/hub/v1;hubpb\242\002\003SHX\252\002\rStanza.Hub.V1\312\002\rStanza\\Hub\\V1\342\002\031Stanza\\Hub\\V1\\GPBMetadata\352\002\017Stanza::Hub::V1'
  _globals['_HEALTHSERVICE'].methods_by_name['QueryGuardHealth']._options = None
  _globals['_HEALTHSERVICE'].methods_by_name['QueryGuardHealth']._serialized_options = b'\222A\257\001\022\020Get Guard Health\032aUsed by SDK to allow developers to make decisions about graceful degradation of backend services.J8\n\003200\0221\n\002OK\022+\n)\032\'.stanza.hub.v1.QueryGuardHealthResponse\202\323\344\223\002\025\"\020/v1/health/guard:\001*'
  _globals['_QUERYGUARDHEALTHREQUEST']._serialized_start=152
  _globals['_QUERYGUARDHEALTHREQUEST']._serialized_end=305
  _globals['_QUERYGUARDHEALTHRESPONSE']._serialized_start=307
  _globals['_QUERYGUARDHEALTHRESPONSE']._serialized_end=380
  _globals['_HEALTHSERVICE']._serialized_start=383
  _globals['_HEALTHSERVICE']._serialized_end=709
# @@protoc_insertion_point(module_scope)



import json
from typing import Any, Mapping


class HubError(Exception):
    """Generic HubError that is a base class of all Hub error exceptions."""

    def __init__(self, code: int, message: str, details: Mapping[str, Any]) -> None:
        super().__init__(message)

        self.code = code
        self.details = details


def hub_error(error: Exception):
    """Construct a Hub error exception associated with the response code."""

    try:
        response = json.loads(error.body)
    except json.JSONDecodeError as exc:
        raise error from exc

    # TODO: Throw different errors based off of the code that we get?
    return HubError(response["code"], response["message"], response.get("details", {}))



import logging
import threading
from typing import Dict, Optional, cast

from getstanza.configuration import StanzaConfiguration
from getstanza.guard import Guard
from getstanza.hub import StanzaHub
from typing_extensions import Self


class StanzaClient:
    """
    SDK client that assists with integrating with Hub and managing the active
    service and guard configurations.
    """

    __instance: Optional[Self] = None
    __lock = threading.Lock()

    @property
    def config(self) -> StanzaConfiguration:
        return self.__config

    @property
    def hub(self) -> Optional[StanzaHub]:
        return self.__hub

    @classmethod
    def getInstance(cls) -> Self:
        """Returns the global instance of the Stanza SDK client."""

        if cls.__instance is None:
            raise RuntimeError("The Stanza SDK has not yet been initialized")

        return cast(Self, cls.__instance)

    def __init__(self, config: StanzaConfiguration):
        """Initializes the Stanza SDK. This can only happen once."""

        logging.debug("Initializing Stanza")

        self.__config = config
        self.__hub = None

        if StanzaClient.__instance is None:
            with StanzaClient.__lock:
                if StanzaClient.__instance is None:
                    StanzaClient.__instance = self
                    self.__start_poller()
                    return

        raise RuntimeError("The Stanza SDK client has already been initialized")

    def __start_poller(self):
        """Initialize the Hub worker and start it on a new thread."""

        def target():
            self.__hub = StanzaHub(self.__config)
            self.__hub.start_poller()

        # We don't worry about cleaning this up since the SDK client instance
        # persists for the lifetime of the process, and only one can exist.
        # Being marked as a daemon thread, this thread will not stop signals
        # from killing the process.
        threading.Thread(target=target, daemon=True).start()

    async def guard(
        self,
        guard_name: str,
        feature: Optional[str] = None,
        priority_boost: Optional[int] = None,
        default_weight: Optional[float] = None,
        tags: Optional[Dict[str, str]] = None,
    ) -> Guard:
        """Initialize a guard and fetch its configuration if not cached."""

        if self.__hub is None:
            raise RuntimeError("The Stanza SDK client hasn't been initialized yet")

        guard = Guard(
            self.__hub,
            guard_name,
            feature_name=feature,
            priority_boost=priority_boost,
            default_weight=default_weight,
            tags=tags,
        )
        await guard.run()

        return guard



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stanza/hub/v1/common.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1astanza/hub/v1/common.proto\x12\rstanza.hub.v1\x1a\x1fgoogle/api/field_behavior.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\"y\n\rGuardSelector\x12&\n\x0b\x65nvironment\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\x0b\x65nvironment\x12\x18\n\x04name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x02R\x04name\x12&\n\x04tags\x18\x06 \x03(\x0b\x32\x12.stanza.hub.v1.TagR\x04tags\"w\n\x0f\x46\x65\x61tureSelector\x12&\n\x0b\x65nvironment\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\x0b\x65nvironment\x12\x14\n\x05names\x18\x02 \x03(\tR\x05names\x12&\n\x04tags\x18\x06 \x03(\x0b\x32\x12.stanza.hub.v1.TagR\x04tags\"\xc4\x01\n\x14GuardFeatureSelector\x12&\n\x0b\x65nvironment\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\x0b\x65nvironment\x12#\n\nguard_name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x02R\tguardName\x12&\n\x0c\x66\x65\x61ture_name\x18\x03 \x01(\tH\x00R\x0b\x66\x65\x61tureName\x88\x01\x01\x12&\n\x04tags\x18\x06 \x03(\x0b\x32\x12.stanza.hub.v1.TagR\x04tagsB\x0f\n\r_feature_name\"\xe3\x01\n\x14GuardServiceSelector\x12&\n\x0b\x65nvironment\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\x0b\x65nvironment\x12#\n\nguard_name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x02R\tguardName\x12\'\n\x0cservice_name\x18\x03 \x01(\tB\x04\xe2\x41\x01\x02R\x0bserviceName\x12-\n\x0fservice_release\x18\x04 \x01(\tB\x04\xe2\x41\x01\x02R\x0eserviceRelease\x12&\n\x04tags\x18\x06 \x03(\x0b\x32\x12.stanza.hub.v1.TagR\x04tags\"\xa6\x01\n\x0fServiceSelector\x12&\n\x0b\x65nvironment\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\x0b\x65nvironment\x12\x18\n\x04name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x02R\x04name\x12\x1d\n\x07release\x18\x03 \x01(\tH\x00R\x07release\x88\x01\x01\x12&\n\x04tags\x18\x06 \x03(\x0b\x32\x12.stanza.hub.v1.TagR\x04tagsB\n\n\x08_release\"]\n\x10HealthByPriority\x12\x1a\n\x08priority\x18\x01 \x01(\rR\x08priority\x12-\n\x06health\x18\x02 \x01(\x0e\x32\x15.stanza.hub.v1.HealthR\x06health\"-\n\x03Tag\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value*U\n\x06Health\x12\x16\n\x12HEALTH_UNSPECIFIED\x10\x00\x12\r\n\tHEALTH_OK\x10\x01\x12\x13\n\x0fHEALTH_OVERLOAD\x10\x02\x12\x0f\n\x0bHEALTH_DOWN\x10\x03*E\n\x05State\x12\x15\n\x11STATE_UNSPECIFIED\x10\x00\x12\x11\n\rSTATE_ENABLED\x10\x01\x12\x12\n\x0eSTATE_DISABLED\x10\x02*\x95\x01\n\x06\x43onfig\x12\x16\n\x12\x43ONFIG_UNSPECIFIED\x10\x00\x12\x14\n\x10\x43ONFIG_CACHED_OK\x10\x01\x12\x15\n\x11\x43ONFIG_FETCHED_OK\x10\x02\x12\x14\n\x10\x43ONFIG_NOT_FOUND\x10\x03\x12\x16\n\x12\x43ONFIG_FETCH_ERROR\x10\x04\x12\x18\n\x14\x43ONFIG_FETCH_TIMEOUT\x10\x05*\x9b\x01\n\x05Local\x12\x15\n\x11LOCAL_UNSPECIFIED\x10\x00\x12\x17\n\x13LOCAL_NOT_SUPPORTED\x10\x01\x12\x12\n\x0eLOCAL_NOT_EVAL\x10\x02\x12\x17\n\x13LOCAL_EVAL_DISABLED\x10\x03\x12\x11\n\rLOCAL_ALLOWED\x10\x04\x12\x11\n\rLOCAL_BLOCKED\x10\x05\x12\x0f\n\x0bLOCAL_ERROR\x10\x06*\xab\x01\n\x05Token\x12\x15\n\x11TOKEN_UNSPECIFIED\x10\x00\x12\x12\n\x0eTOKEN_NOT_EVAL\x10\x01\x12\x17\n\x13TOKEN_EVAL_DISABLED\x10\x02\x12\x13\n\x0fTOKEN_NOT_VALID\x10\x03\x12\x0f\n\x0bTOKEN_VALID\x10\x04\x12\x1a\n\x16TOKEN_VALIDATION_ERROR\x10\x05\x12\x1c\n\x18TOKEN_VALIDATION_TIMEOUT\x10\x06*\xac\x01\n\x05Quota\x12\x15\n\x11QUOTA_UNSPECIFIED\x10\x00\x12\x12\n\x0eQUOTA_NOT_EVAL\x10\x01\x12\x17\n\x13QUOTA_EVAL_DISABLED\x10\x02\x12\x15\n\x11QUOTA_LOCAL_ERROR\x10\x03\x12\x11\n\rQUOTA_BLOCKED\x10\x04\x12\x11\n\rQUOTA_GRANTED\x10\x05\x12\x0f\n\x0bQUOTA_ERROR\x10\x06\x12\x11\n\rQUOTA_TIMEOUT\x10\x07*C\n\x04Mode\x12\x14\n\x10MODE_UNSPECIFIED\x10\x00\x12\x0f\n\x0bMODE_NORMAL\x10\x01\x12\x14\n\x10MODE_REPORT_ONLY\x10\x02\x42\x80\x05\n\x11\x63om.stanza.hub.v1B\x0b\x43ommonProtoP\x01Z\x13stanza/hub/v1;hubpb\xa2\x02\x03SHX\xaa\x02\rStanza.Hub.V1\xca\x02\rStanza\\Hub\\V1\xe2\x02\x19Stanza\\Hub\\V1\\GPBMetadata\xea\x02\x0fStanza::Hub::V1\x92\x41\xf1\x03\x12O\n\x0eStanza Hub API\"8\n\x06Stanza\x12\x16https://stanza.systems\x1a\x16support@stanza.systems2\x03\x31.0*\x01\x02\x32\x10\x61pplication/json:\x10\x61pplication/jsonRU\n\x03\x34\x32\x39\x12N\n/Too Many Requests, try again after Retry-After.\x12\x1b\n\x19\x1a\x17#/definitions/rpcStatusR;\n\x03\x35\x30\x30\x12\x34\n\x15Internal Server Error\x12\x1b\n\x19\x1a\x17#/definitions/rpcStatusRW\n\x03\x35\x30\x33\x12P\n1Service Unavailable, try again after Retry-After.\x12\x1b\n\x19\x1a\x17#/definitions/rpcStatusZ\"\n \n\nApiKeyAuth\x12\x12\x08\x02\x1a\x0cX-Stanza-Key \x02\x62\x10\n\x0e\n\nApiKeyAuth\x12\x00rT\n\x0eStanza Hub API\x12\x42https://stanza.stoplight.io/docs/apis/291a2cff9d156-stanza-hub-apib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stanza.hub.v1.common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\021com.stanza.hub.v1B\013CommonProtoP\001Z\023stanza/hub/v1;hubpb\242\002\003SHX\252\002\rStanza.Hub.V1\312\002\rStanza\\Hub\\V1\342\002\031Stanza\\Hub\\V1\\GPBMetadata\352\002\017Stanza::Hub::V1\222A\361\003\022O\n\016Stanza Hub API\"8\n\006Stanza\022\026https://stanza.systems\032\026support@stanza.systems2\0031.0*\001\0022\020application/json:\020application/jsonRU\n\003429\022N\n/Too Many Requests, try again after Retry-After.\022\033\n\031\032\027#/definitions/rpcStatusR;\n\003500\0224\n\025Internal Server Error\022\033\n\031\032\027#/definitions/rpcStatusRW\n\003503\022P\n1Service Unavailable, try again after Retry-After.\022\033\n\031\032\027#/definitions/rpcStatusZ\"\n \n\nApiKeyAuth\022\022\010\002\032\014X-Stanza-Key \002b\020\n\016\n\nApiKeyAuth\022\000rT\n\016Stanza Hub API\022Bhttps://stanza.stoplight.io/docs/apis/291a2cff9d156-stanza-hub-api'
  _globals['_GUARDSELECTOR'].fields_by_name['environment']._options = None
  _globals['_GUARDSELECTOR'].fields_by_name['environment']._serialized_options = b'\342A\001\002'
  _globals['_GUARDSELECTOR'].fields_by_name['name']._options = None
  _globals['_GUARDSELECTOR'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_FEATURESELECTOR'].fields_by_name['environment']._options = None
  _globals['_FEATURESELECTOR'].fields_by_name['environment']._serialized_options = b'\342A\001\002'
  _globals['_GUARDFEATURESELECTOR'].fields_by_name['environment']._options = None
  _globals['_GUARDFEATURESELECTOR'].fields_by_name['environment']._serialized_options = b'\342A\001\002'
  _globals['_GUARDFEATURESELECTOR'].fields_by_name['guard_name']._options = None
  _globals['_GUARDFEATURESELECTOR'].fields_by_name['guard_name']._serialized_options = b'\342A\001\002'
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['environment']._options = None
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['environment']._serialized_options = b'\342A\001\002'
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['guard_name']._options = None
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['guard_name']._serialized_options = b'\342A\001\002'
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['service_name']._options = None
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['service_name']._serialized_options = b'\342A\001\002'
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['service_release']._options = None
  _globals['_GUARDSERVICESELECTOR'].fields_by_name['service_release']._serialized_options = b'\342A\001\002'
  _globals['_SERVICESELECTOR'].fields_by_name['environment']._options = None
  _globals['_SERVICESELECTOR'].fields_by_name['environment']._serialized_options = b'\342A\001\002'
  _globals['_SERVICESELECTOR'].fields_by_name['name']._options = None
  _globals['_SERVICESELECTOR'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_HEALTH']._serialized_start=1110
  _globals['_HEALTH']._serialized_end=1195
  _globals['_STATE']._serialized_start=1197
  _globals['_STATE']._serialized_end=1266
  _globals['_CONFIG']._serialized_start=1269
  _globals['_CONFIG']._serialized_end=1418
  _globals['_LOCAL']._serialized_start=1421
  _globals['_LOCAL']._serialized_end=1576
  _globals['_TOKEN']._serialized_start=1579
  _globals['_TOKEN']._serialized_end=1750
  _globals['_QUOTA']._serialized_start=1753
  _globals['_QUOTA']._serialized_end=1925
  _globals['_MODE']._serialized_start=1927
  _globals['_MODE']._serialized_end=1994
  _globals['_GUARDSELECTOR']._serialized_start=126
  _globals['_GUARDSELECTOR']._serialized_end=247
  _globals['_FEATURESELECTOR']._serialized_start=249
  _globals['_FEATURESELECTOR']._serialized_end=368
  _globals['_GUARDFEATURESELECTOR']._serialized_start=371
  _globals['_GUARDFEATURESELECTOR']._serialized_end=567
  _globals['_GUARDSERVICESELECTOR']._serialized_start=570
  _globals['_GUARDSERVICESELECTOR']._serialized_end=797
  _globals['_SERVICESELECTOR']._serialized_start=800
  _globals['_SERVICESELECTOR']._serialized_end=966
  _globals['_HEALTHBYPRIORITY']._serialized_start=968
  _globals['_HEALTHBYPRIORITY']._serialized_end=1061
  _globals['_TAG']._serialized_start=1063
  _globals['_TAG']._serialized_end=1108
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stanza/hub/v1/config.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from stanza.hub.v1 import common_pb2 as stanza_dot_hub_dot_v1_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1astanza/hub/v1/config.proto\x12\rstanza.hub.v1\x1a\x1cgoogle/api/annotations.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a\x1astanza/hub/v1/common.proto\"\x91\x01\n\x15GetGuardConfigRequest\x12&\n\x0cversion_seen\x18\x01 \x01(\tH\x00R\x0bversionSeen\x88\x01\x01\x12?\n\x08selector\x18\x02 \x01(\x0b\x32#.stanza.hub.v1.GuardServiceSelectorR\x08selectorB\x0f\n\r_version_seen\"\x90\x01\n\x16GetGuardConfigResponse\x12\x18\n\x07version\x18\x01 \x01(\tR\x07version\x12(\n\x10\x63onfig_data_sent\x18\x02 \x01(\x08R\x0e\x63onfigDataSent\x12\x32\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x1a.stanza.hub.v1.GuardConfigR\x06\x63onfig\"\xa6\x01\n\x0bGuardConfig\x12\x36\n\x17validate_ingress_tokens\x18\x01 \x01(\x08R\x15validateIngressTokens\x12\x1f\n\x0b\x63heck_quota\x18\x05 \x01(\x08R\ncheckQuota\x12\x1d\n\nquota_tags\x18\x06 \x03(\tR\tquotaTags\x12\x1f\n\x0breport_only\x18\x07 \x01(\x08R\nreportOnly\"T\n\x18GetBrowserContextRequest\x12\x38\n\x07\x66\x65\x61ture\x18\x01 \x01(\x0b\x32\x1e.stanza.hub.v1.FeatureSelectorR\x07\x66\x65\x61ture\"b\n\x19GetBrowserContextResponse\x12\x45\n\x0f\x66\x65\x61ture_configs\x18\x02 \x03(\x0b\x32\x1c.stanza.hub.v1.FeatureConfigR\x0e\x66\x65\x61tureConfigs\"Y\n\rFeatureConfig\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12\x34\n\x06\x63onfig\x18\x02 \x01(\x0b\x32\x1c.stanza.hub.v1.BrowserConfigR\x06\x63onfig\"\xf5\x02\n\rBrowserConfig\x12,\n\x0f\x65nabled_percent\x18\x02 \x01(\rH\x00R\x0e\x65nabledPercent\x88\x01\x01\x12\x33\n\x13\x61\x63tion_code_enabled\x18\x03 \x01(\rH\x01R\x11\x61\x63tionCodeEnabled\x88\x01\x01\x12,\n\x0fmessage_enabled\x18\x04 \x01(\tH\x02R\x0emessageEnabled\x88\x01\x01\x12\x35\n\x14\x61\x63tion_code_disabled\x18\x05 \x01(\rH\x03R\x12\x61\x63tionCodeDisabled\x88\x01\x01\x12.\n\x10message_disabled\x18\x06 \x01(\tH\x04R\x0fmessageDisabled\x88\x01\x01\x42\x12\n\x10_enabled_percentB\x16\n\x14_action_code_enabledB\x12\n\x10_message_enabledB\x17\n\x15_action_code_disabledB\x13\n\x11_message_disabled\"\xa6\x01\n\x17GetServiceConfigRequest\x12!\n\x0cversion_seen\x18\x01 \x01(\tR\x0bversionSeen\x12\x38\n\x07service\x18\x02 \x01(\x0b\x32\x1e.stanza.hub.v1.ServiceSelectorR\x07service\x12 \n\tclient_id\x18\x03 \x01(\tH\x00R\x08\x63lientId\x88\x01\x01\x42\x0c\n\n_client_id\"\x94\x01\n\x18GetServiceConfigResponse\x12\x18\n\x07version\x18\x01 \x01(\tR\x07version\x12(\n\x10\x63onfig_data_sent\x18\x02 \x01(\x08R\x0e\x63onfigDataSent\x12\x34\n\x06\x63onfig\x18\x03 \x01(\x0b\x32\x1c.stanza.hub.v1.ServiceConfigR\x06\x63onfig\"\xd4\x02\n\rServiceConfig\x12$\n\x0b\x63ustomer_id\x18\x01 \x01(\tH\x00R\ncustomerId\x88\x01\x01\x12\x42\n\x0ctrace_config\x18\x03 \x01(\x0b\x32\x1a.stanza.hub.v1.TraceConfigH\x01R\x0btraceConfig\x88\x01\x01\x12\x45\n\rmetric_config\x18\x04 \x01(\x0b\x32\x1b.stanza.hub.v1.MetricConfigH\x02R\x0cmetricConfig\x88\x01\x01\x12K\n\x0fsentinel_config\x18\x05 \x01(\x0b\x32\x1d.stanza.hub.v1.SentinelConfigH\x03R\x0esentinelConfig\x88\x01\x01\x42\x0e\n\x0c_customer_idB\x0f\n\r_trace_configB\x10\n\x0e_metric_configB\x12\n\x10_sentinel_config\"\x81\x03\n\x0bTraceConfig\x12(\n\rcollector_url\x18\x01 \x01(\tH\x00R\x0c\x63ollectorUrl\x88\x01\x01\x12\x33\n\x13sample_rate_default\x18\x03 \x01(\x02H\x01R\x11sampleRateDefault\x88\x01\x01\x12@\n\toverrides\x18\x04 \x03(\x0b\x32\".stanza.hub.v1.TraceConfigOverrideR\toverrides\x12T\n\x15header_sample_configs\x18\x05 \x03(\x0b\x32 .stanza.hub.v1.HeaderTraceConfigR\x13headerSampleConfigs\x12Q\n\x14param_sample_configs\x18\x06 \x03(\x0b\x32\x1f.stanza.hub.v1.ParamTraceConfigR\x12paramSampleConfigsB\x10\n\x0e_collector_urlB\x16\n\x14_sample_rate_default\"J\n\x0cMetricConfig\x12(\n\rcollector_url\x18\x01 \x01(\tH\x00R\x0c\x63ollectorUrl\x88\x01\x01\x42\x10\n\x0e_collector_url\"\xc7\x02\n\x0eSentinelConfig\x12?\n\x19\x63ircuitbreaker_rules_json\x18\x01 \x01(\tH\x00R\x17\x63ircuitbreakerRulesJson\x88\x01\x01\x12+\n\x0f\x66low_rules_json\x18\x02 \x01(\tH\x01R\rflowRulesJson\x88\x01\x01\x12\x35\n\x14isolation_rules_json\x18\x03 \x01(\tH\x02R\x12isolationRulesJson\x88\x01\x01\x12/\n\x11system_rules_json\x18\x04 \x01(\tH\x03R\x0fsystemRulesJson\x88\x01\x01\x42\x1c\n\x1a_circuitbreaker_rules_jsonB\x12\n\x10_flow_rules_jsonB\x17\n\x15_isolation_rules_jsonB\x14\n\x12_system_rules_json\"z\n\x13TraceConfigOverride\x12\x1f\n\x0bsample_rate\x18\x01 \x01(\x02R\nsampleRate\x12\x42\n\x0espan_selectors\x18\x02 \x03(\x0b\x32\x1b.stanza.hub.v1.SpanSelectorR\rspanSelectors\"K\n\x0cSpanSelector\x12%\n\x0eotel_attribute\x18\x01 \x01(\tR\rotelAttribute\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value\"\xbd\x01\n\x11HeaderTraceConfig\x12\x42\n\x0espan_selectors\x18\x01 \x03(\x0b\x32\x1b.stanza.hub.v1.SpanSelectorR\rspanSelectors\x12\x30\n\x14request_header_names\x18\x02 \x03(\tR\x12requestHeaderNames\x12\x32\n\x15response_header_names\x18\x03 \x03(\tR\x13responseHeaderNames\"\x7f\n\x10ParamTraceConfig\x12\x42\n\x0espan_selectors\x18\x01 \x03(\x0b\x32\x1b.stanza.hub.v1.SpanSelectorR\rspanSelectors\x12\'\n\x0fparameter_names\x18\x02 \x03(\tR\x0eparameterNames2\xef\x07\n\rConfigService\x12\xfc\x01\n\x0eGetGuardConfig\x12$.stanza.hub.v1.GetGuardConfigRequest\x1a%.stanza.hub.v1.GetGuardConfigResponse\"\x9c\x01\x92\x41~\x12\x10Get Guard Config\x1a\x32Used by SDK to get a Guard Config from Stanza Hub.J6\n\x03\x32\x30\x30\x12/\n\x02OK\x12)\n\'\x1a%.stanza.hub.v1.GetGuardConfigResponse\x82\xd3\xe4\x93\x02\x15\"\x10/v1/config/guard:\x01*\x12\xd0\x03\n\x11GetBrowserContext\x12\'.stanza.hub.v1.GetBrowserContextRequest\x1a(.stanza.hub.v1.GetBrowserContextResponse\"\xe7\x02\x92\x41\xc5\x02\x12\x13Get Browser Context\x1a\x35Used by SDK to get a Browser Context from Stanza Hub.J\x8a\x01\n\x03\x32\x30\x30\x12\x82\x01\n\x02OK\x12,\n*\x1a(.stanza.hub.v1.GetBrowserContextResponse\x1aN\n\x04\x45Tag\x12\x46\n<Unique entity identifier for this specific resource version.\x12\x06stringJj\n\x03\x33\x30\x34\x12\x63\n\x11OK, Not Modified.\x1aN\n\x04\x45Tag\x12\x46\n<Unique entity identifier for this specific resource version.\x12\x06string\x82\xd3\xe4\x93\x02\x18\"\x13/v1/context/browser:\x01*\x12\x8b\x02\n\x10GetServiceConfig\x12&.stanza.hub.v1.GetServiceConfigRequest\x1a\'.stanza.hub.v1.GetServiceConfigResponse\"\xa5\x01\x92\x41\x84\x01\x12\x12Get Service Config\x1a\x34Used by SDK to get a Service Config from Stanza Hub.J8\n\x03\x32\x30\x30\x12\x31\n\x02OK\x12+\n)\x1a\'.stanza.hub.v1.GetServiceConfigResponse\x82\xd3\xe4\x93\x02\x17\"\x12/v1/config/service:\x01*B\x8b\x01\n\x11\x63om.stanza.hub.v1B\x0b\x43onfigProtoP\x01Z\x13stanza/hub/v1;hubpb\xa2\x02\x03SHX\xaa\x02\rStanza.Hub.V1\xca\x02\rStanza\\Hub\\V1\xe2\x02\x19Stanza\\Hub\\V1\\GPBMetadata\xea\x02\x0fStanza::Hub::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stanza.hub.v1.config_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\021com.stanza.hub.v1B\013ConfigProtoP\001Z\023stanza/hub/v1;hubpb\242\002\003SHX\252\002\rStanza.Hub.V1\312\002\rStanza\\Hub\\V1\342\002\031Stanza\\Hub\\V1\\GPBMetadata\352\002\017Stanza::Hub::V1'
  _globals['_CONFIGSERVICE'].methods_by_name['GetGuardConfig']._options = None
  _globals['_CONFIGSERVICE'].methods_by_name['GetGuardConfig']._serialized_options = b'\222A~\022\020Get Guard Config\0322Used by SDK to get a Guard Config from Stanza Hub.J6\n\003200\022/\n\002OK\022)\n\'\032%.stanza.hub.v1.GetGuardConfigResponse\202\323\344\223\002\025\"\020/v1/config/guard:\001*'
  _globals['_CONFIGSERVICE'].methods_by_name['GetBrowserContext']._options = None
  _globals['_CONFIGSERVICE'].methods_by_name['GetBrowserContext']._serialized_options = b'\222A\305\002\022\023Get Browser Context\0325Used by SDK to get a Browser Context from Stanza Hub.J\212\001\n\003200\022\202\001\n\002OK\022,\n*\032(.stanza.hub.v1.GetBrowserContextResponse\032N\n\004ETag\022F\n<Unique entity identifier for this specific resource version.\022\006stringJj\n\003304\022c\n\021OK, Not Modified.\032N\n\004ETag\022F\n<Unique entity identifier for this specific resource version.\022\006string\202\323\344\223\002\030\"\023/v1/context/browser:\001*'
  _globals['_CONFIGSERVICE'].methods_by_name['GetServiceConfig']._options = None
  _globals['_CONFIGSERVICE'].methods_by_name['GetServiceConfig']._serialized_options = b'\222A\204\001\022\022Get Service Config\0324Used by SDK to get a Service Config from Stanza Hub.J8\n\003200\0221\n\002OK\022+\n)\032\'.stanza.hub.v1.GetServiceConfigResponse\202\323\344\223\002\027\"\022/v1/config/service:\001*'
  _globals['_GETGUARDCONFIGREQUEST']._serialized_start=152
  _globals['_GETGUARDCONFIGREQUEST']._serialized_end=297
  _globals['_GETGUARDCONFIGRESPONSE']._serialized_start=300
  _globals['_GETGUARDCONFIGRESPONSE']._serialized_end=444
  _globals['_GUARDCONFIG']._serialized_start=447
  _globals['_GUARDCONFIG']._serialized_end=613
  _globals['_GETBROWSERCONTEXTREQUEST']._serialized_start=615
  _globals['_GETBROWSERCONTEXTREQUEST']._serialized_end=699
  _globals['_GETBROWSERCONTEXTRESPONSE']._serialized_start=701
  _globals['_GETBROWSERCONTEXTRESPONSE']._serialized_end=799
  _globals['_FEATURECONFIG']._serialized_start=801
  _globals['_FEATURECONFIG']._serialized_end=890
  _globals['_BROWSERCONFIG']._serialized_start=893
  _globals['_BROWSERCONFIG']._serialized_end=1266
  _globals['_GETSERVICECONFIGREQUEST']._serialized_start=1269
  _globals['_GETSERVICECONFIGREQUEST']._serialized_end=1435
  _globals['_GETSERVICECONFIGRESPONSE']._serialized_start=1438
  _globals['_GETSERVICECONFIGRESPONSE']._serialized_end=1586
  _globals['_SERVICECONFIG']._serialized_start=1589
  _globals['_SERVICECONFIG']._serialized_end=1929
  _globals['_TRACECONFIG']._serialized_start=1932
  _globals['_TRACECONFIG']._serialized_end=2317
  _globals['_METRICCONFIG']._serialized_start=2319
  _globals['_METRICCONFIG']._serialized_end=2393
  _globals['_SENTINELCONFIG']._serialized_start=2396
  _globals['_SENTINELCONFIG']._serialized_end=2723
  _globals['_TRACECONFIGOVERRIDE']._serialized_start=2725
  _globals['_TRACECONFIGOVERRIDE']._serialized_end=2847
  _globals['_SPANSELECTOR']._serialized_start=2849
  _globals['_SPANSELECTOR']._serialized_end=2924
  _globals['_HEADERTRACECONFIG']._serialized_start=2927
  _globals['_HEADERTRACECONFIG']._serialized_end=3116
  _globals['_PARAMTRACECONFIG']._serialized_start=3118
  _globals['_PARAMTRACECONFIG']._serialized_end=3245
  _globals['_CONFIGSERVICE']._serialized_start=3248
  _globals['_CONFIGSERVICE']._serialized_end=4255
# @@protoc_insertion_point(module_scope)



from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from stanza.hub.v1 import common_pb2 as _common_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class QueryMode(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    QUERY_MODE_UNSPECIFIED: _ClassVar[QueryMode]
    QUERY_MODE_SUM: _ClassVar[QueryMode]
    QUERY_MODE_REPORT: _ClassVar[QueryMode]
QUERY_MODE_UNSPECIFIED: QueryMode
QUERY_MODE_SUM: QueryMode
QUERY_MODE_REPORT: QueryMode

class GetUsageRequest(_message.Message):
    __slots__ = ("environment", "guard", "guard_query_mode", "start_ts", "end_ts", "apikey", "feature", "feature_query_mode", "service", "service_query_mode", "priority", "priority_query_mode", "report_tags", "tags", "report_all_tags", "step")
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    GUARD_FIELD_NUMBER: _ClassVar[int]
    GUARD_QUERY_MODE_FIELD_NUMBER: _ClassVar[int]
    START_TS_FIELD_NUMBER: _ClassVar[int]
    END_TS_FIELD_NUMBER: _ClassVar[int]
    APIKEY_FIELD_NUMBER: _ClassVar[int]
    FEATURE_FIELD_NUMBER: _ClassVar[int]
    FEATURE_QUERY_MODE_FIELD_NUMBER: _ClassVar[int]
    SERVICE_FIELD_NUMBER: _ClassVar[int]
    SERVICE_QUERY_MODE_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_QUERY_MODE_FIELD_NUMBER: _ClassVar[int]
    REPORT_TAGS_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    REPORT_ALL_TAGS_FIELD_NUMBER: _ClassVar[int]
    STEP_FIELD_NUMBER: _ClassVar[int]
    environment: str
    guard: str
    guard_query_mode: QueryMode
    start_ts: _timestamp_pb2.Timestamp
    end_ts: _timestamp_pb2.Timestamp
    apikey: str
    feature: str
    feature_query_mode: QueryMode
    service: str
    service_query_mode: QueryMode
    priority: int
    priority_query_mode: QueryMode
    report_tags: _containers.RepeatedScalarFieldContainer[str]
    tags: _containers.RepeatedCompositeFieldContainer[_common_pb2.Tag]
    report_all_tags: bool
    step: str
    def __init__(self, environment: _Optional[str] = ..., guard: _Optional[str] = ..., guard_query_mode: _Optional[_Union[QueryMode, str]] = ..., start_ts: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end_ts: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., apikey: _Optional[str] = ..., feature: _Optional[str] = ..., feature_query_mode: _Optional[_Union[QueryMode, str]] = ..., service: _Optional[str] = ..., service_query_mode: _Optional[_Union[QueryMode, str]] = ..., priority: _Optional[int] = ..., priority_query_mode: _Optional[_Union[QueryMode, str]] = ..., report_tags: _Optional[_Iterable[str]] = ..., tags: _Optional[_Iterable[_Union[_common_pb2.Tag, _Mapping]]] = ..., report_all_tags: bool = ..., step: _Optional[str] = ...) -> None: ...

class GetUsageResponse(_message.Message):
    __slots__ = ("result",)
    RESULT_FIELD_NUMBER: _ClassVar[int]
    result: _containers.RepeatedCompositeFieldContainer[UsageTimeseries]
    def __init__(self, result: _Optional[_Iterable[_Union[UsageTimeseries, _Mapping]]] = ...) -> None: ...

class UsageTimeseries(_message.Message):
    __slots__ = ("data", "feature", "priority", "tags", "guard", "service")
    DATA_FIELD_NUMBER: _ClassVar[int]
    FEATURE_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    GUARD_FIELD_NUMBER: _ClassVar[int]
    SERVICE_FIELD_NUMBER: _ClassVar[int]
    data: _containers.RepeatedCompositeFieldContainer[UsageTSDataPoint]
    feature: str
    priority: int
    tags: _containers.RepeatedCompositeFieldContainer[_common_pb2.Tag]
    guard: str
    service: str
    def __init__(self, data: _Optional[_Iterable[_Union[UsageTSDataPoint, _Mapping]]] = ..., feature: _Optional[str] = ..., priority: _Optional[int] = ..., tags: _Optional[_Iterable[_Union[_common_pb2.Tag, _Mapping]]] = ..., guard: _Optional[str] = ..., service: _Optional[str] = ...) -> None: ...

class UsageTSDataPoint(_message.Message):
    __slots__ = ("start_ts", "end_ts", "granted", "granted_weight", "not_granted", "not_granted_weight", "be_burst", "be_burst_weight", "parent_reject", "parent_reject_weight")
    START_TS_FIELD_NUMBER: _ClassVar[int]
    END_TS_FIELD_NUMBER: _ClassVar[int]
    GRANTED_FIELD_NUMBER: _ClassVar[int]
    GRANTED_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    NOT_GRANTED_FIELD_NUMBER: _ClassVar[int]
    NOT_GRANTED_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    BE_BURST_FIELD_NUMBER: _ClassVar[int]
    BE_BURST_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    PARENT_REJECT_FIELD_NUMBER: _ClassVar[int]
    PARENT_REJECT_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    start_ts: _timestamp_pb2.Timestamp
    end_ts: _timestamp_pb2.Timestamp
    granted: int
    granted_weight: float
    not_granted: int
    not_granted_weight: float
    be_burst: int
    be_burst_weight: float
    parent_reject: int
    parent_reject_weight: float
    def __init__(self, start_ts: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end_ts: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., granted: _Optional[int] = ..., granted_weight: _Optional[float] = ..., not_granted: _Optional[int] = ..., not_granted_weight: _Optional[float] = ..., be_burst: _Optional[int] = ..., be_burst_weight: _Optional[float] = ..., parent_reject: _Optional[int] = ..., parent_reject_weight: _Optional[float] = ...) -> None: ...



from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from protoc_gen_openapiv2.options import annotations_pb2 as _annotations_pb2_1
from stanza.hub.v1 import common_pb2 as _common_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Reason(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = ()
    REASON_UNSPECIFIED: _ClassVar[Reason]
    REASON_SUFFICIENT_QUOTA: _ClassVar[Reason]
    REASON_INSUFFICIENT_QUOTA: _ClassVar[Reason]
    REASON_INSUFFICIENT_QUOTA_PARENT: _ClassVar[Reason]
    REASON_BURST: _ClassVar[Reason]
    REASON_BEST_EFFORT: _ClassVar[Reason]
REASON_UNSPECIFIED: Reason
REASON_SUFFICIENT_QUOTA: Reason
REASON_INSUFFICIENT_QUOTA: Reason
REASON_INSUFFICIENT_QUOTA_PARENT: Reason
REASON_BURST: Reason
REASON_BEST_EFFORT: Reason

class GetTokenRequest(_message.Message):
    __slots__ = ("selector", "client_id", "priority_boost", "weight")
    SELECTOR_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_BOOST_FIELD_NUMBER: _ClassVar[int]
    WEIGHT_FIELD_NUMBER: _ClassVar[int]
    selector: _common_pb2.GuardFeatureSelector
    client_id: str
    priority_boost: int
    weight: float
    def __init__(self, selector: _Optional[_Union[_common_pb2.GuardFeatureSelector, _Mapping]] = ..., client_id: _Optional[str] = ..., priority_boost: _Optional[int] = ..., weight: _Optional[float] = ...) -> None: ...

class GetTokenResponse(_message.Message):
    __slots__ = ("granted", "token", "reason", "mode")
    GRANTED_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    MODE_FIELD_NUMBER: _ClassVar[int]
    granted: bool
    token: str
    reason: Reason
    mode: _common_pb2.Mode
    def __init__(self, granted: bool = ..., token: _Optional[str] = ..., reason: _Optional[_Union[Reason, str]] = ..., mode: _Optional[_Union[_common_pb2.Mode, str]] = ...) -> None: ...

class GetTokenLeaseRequest(_message.Message):
    __slots__ = ("selector", "client_id", "priority_boost", "default_weight")
    SELECTOR_FIELD_NUMBER: _ClassVar[int]
    CLIENT_ID_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_BOOST_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    selector: _common_pb2.GuardFeatureSelector
    client_id: str
    priority_boost: int
    default_weight: float
    def __init__(self, selector: _Optional[_Union[_common_pb2.GuardFeatureSelector, _Mapping]] = ..., client_id: _Optional[str] = ..., priority_boost: _Optional[int] = ..., default_weight: _Optional[float] = ...) -> None: ...

class GetTokenLeaseResponse(_message.Message):
    __slots__ = ("granted", "leases")
    GRANTED_FIELD_NUMBER: _ClassVar[int]
    LEASES_FIELD_NUMBER: _ClassVar[int]
    granted: bool
    leases: _containers.RepeatedCompositeFieldContainer[TokenLease]
    def __init__(self, granted: bool = ..., leases: _Optional[_Iterable[_Union[TokenLease, _Mapping]]] = ...) -> None: ...

class TokenLease(_message.Message):
    __slots__ = ("duration_msec", "token", "feature", "priority_boost", "weight", "reason", "expires_at", "mode")
    DURATION_MSEC_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    FEATURE_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_BOOST_FIELD_NUMBER: _ClassVar[int]
    WEIGHT_FIELD_NUMBER: _ClassVar[int]
    REASON_FIELD_NUMBER: _ClassVar[int]
    EXPIRES_AT_FIELD_NUMBER: _ClassVar[int]
    MODE_FIELD_NUMBER: _ClassVar[int]
    duration_msec: int
    token: str
    feature: str
    priority_boost: int
    weight: float
    reason: Reason
    expires_at: _timestamp_pb2.Timestamp
    mode: _common_pb2.Mode
    def __init__(self, duration_msec: _Optional[int] = ..., token: _Optional[str] = ..., feature: _Optional[str] = ..., priority_boost: _Optional[int] = ..., weight: _Optional[float] = ..., reason: _Optional[_Union[Reason, str]] = ..., expires_at: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., mode: _Optional[_Union[_common_pb2.Mode, str]] = ...) -> None: ...

class SetTokenLeaseConsumedRequest(_message.Message):
    __slots__ = ("tokens", "weight_correction", "environment")
    TOKENS_FIELD_NUMBER: _ClassVar[int]
    WEIGHT_CORRECTION_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    tokens: _containers.RepeatedScalarFieldContainer[str]
    weight_correction: float
    environment: str
    def __init__(self, tokens: _Optional[_Iterable[str]] = ..., weight_correction: _Optional[float] = ..., environment: _Optional[str] = ...) -> None: ...

class SetTokenLeaseConsumedResponse(_message.Message):
    __slots__ = ()
    def __init__(self) -> None: ...

class ValidateTokenRequest(_message.Message):
    __slots__ = ("tokens",)
    TOKENS_FIELD_NUMBER: _ClassVar[int]
    tokens: _containers.RepeatedCompositeFieldContainer[TokenInfo]
    def __init__(self, tokens: _Optional[_Iterable[_Union[TokenInfo, _Mapping]]] = ...) -> None: ...

class TokenInfo(_message.Message):
    __slots__ = ("token", "guard")
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    GUARD_FIELD_NUMBER: _ClassVar[int]
    token: str
    guard: _common_pb2.GuardSelector
    def __init__(self, token: _Optional[str] = ..., guard: _Optional[_Union[_common_pb2.GuardSelector, _Mapping]] = ...) -> None: ...

class ValidateTokenResponse(_message.Message):
    __slots__ = ("valid", "tokens_valid")
    VALID_FIELD_NUMBER: _ClassVar[int]
    TOKENS_VALID_FIELD_NUMBER: _ClassVar[int]
    valid: bool
    tokens_valid: _containers.RepeatedCompositeFieldContainer[TokenValid]
    def __init__(self, valid: bool = ..., tokens_valid: _Optional[_Iterable[_Union[TokenValid, _Mapping]]] = ...) -> None: ...

class TokenValid(_message.Message):
    __slots__ = ("token", "valid")
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    VALID_FIELD_NUMBER: _ClassVar[int]
    token: str
    valid: bool
    def __init__(self, token: _Optional[str] = ..., valid: bool = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stanza/hub/v1/quota.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from protoc_gen_openapiv2.options import annotations_pb2 as protoc__gen__openapiv2_dot_options_dot_annotations__pb2
from stanza.hub.v1 import common_pb2 as stanza_dot_hub_dot_v1_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19stanza/hub/v1/quota.proto\x12\rstanza.hub.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a.protoc-gen-openapiv2/options/annotations.proto\x1a\x1astanza/hub/v1/common.proto\"\xef\x01\n\x0fGetTokenRequest\x12\x45\n\x08selector\x18\x01 \x01(\x0b\x32#.stanza.hub.v1.GuardFeatureSelectorB\x04\xe2\x41\x01\x02R\x08selector\x12 \n\tclient_id\x18\x04 \x01(\tH\x00R\x08\x63lientId\x88\x01\x01\x12*\n\x0epriority_boost\x18\x05 \x01(\x05H\x01R\rpriorityBoost\x88\x01\x01\x12\x1b\n\x06weight\x18\x07 \x01(\x02H\x02R\x06weight\x88\x01\x01\x42\x0c\n\n_client_idB\x11\n\x0f_priority_boostB\t\n\x07_weight\"\xcd\x01\n\x10GetTokenResponse\x12\x1e\n\x07granted\x18\x01 \x01(\x08\x42\x04\xe2\x41\x01\x02R\x07granted\x12\x19\n\x05token\x18\x02 \x01(\tH\x00R\x05token\x88\x01\x01\x12\x32\n\x06reason\x18\x03 \x01(\x0e\x32\x15.stanza.hub.v1.ReasonH\x01R\x06reason\x88\x01\x01\x12,\n\x04mode\x18\x04 \x01(\x0e\x32\x13.stanza.hub.v1.ModeH\x02R\x04mode\x88\x01\x01\x42\x08\n\x06_tokenB\t\n\x07_reasonB\x07\n\x05_mode\"\x8b\x02\n\x14GetTokenLeaseRequest\x12\x45\n\x08selector\x18\x01 \x01(\x0b\x32#.stanza.hub.v1.GuardFeatureSelectorB\x04\xe2\x41\x01\x02R\x08selector\x12 \n\tclient_id\x18\x04 \x01(\tH\x00R\x08\x63lientId\x88\x01\x01\x12*\n\x0epriority_boost\x18\x05 \x01(\x05H\x01R\rpriorityBoost\x88\x01\x01\x12*\n\x0e\x64\x65\x66\x61ult_weight\x18\x07 \x01(\x02H\x02R\rdefaultWeight\x88\x01\x01\x42\x0c\n\n_client_idB\x11\n\x0f_priority_boostB\x11\n\x0f_default_weight\"j\n\x15GetTokenLeaseResponse\x12\x1e\n\x07granted\x18\x01 \x01(\x08\x42\x04\xe2\x41\x01\x02R\x07granted\x12\x31\n\x06leases\x18\x02 \x03(\x0b\x32\x19.stanza.hub.v1.TokenLeaseR\x06leases\"\xd5\x02\n\nTokenLease\x12#\n\rduration_msec\x18\x01 \x01(\x05R\x0c\x64urationMsec\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\x12\x18\n\x07\x66\x65\x61ture\x18\x03 \x01(\tR\x07\x66\x65\x61ture\x12%\n\x0epriority_boost\x18\x04 \x01(\x05R\rpriorityBoost\x12\x16\n\x06weight\x18\x05 \x01(\x02R\x06weight\x12-\n\x06reason\x18\x06 \x01(\x0e\x32\x15.stanza.hub.v1.ReasonR\x06reason\x12>\n\nexpires_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00R\texpiresAt\x88\x01\x01\x12,\n\x04mode\x18\x08 \x01(\x0e\x32\x13.stanza.hub.v1.ModeH\x01R\x04mode\x88\x01\x01\x42\r\n\x0b_expires_atB\x07\n\x05_mode\"\xa6\x01\n\x1cSetTokenLeaseConsumedRequest\x12\x1c\n\x06tokens\x18\x01 \x03(\tB\x04\xe2\x41\x01\x02R\x06tokens\x12\x30\n\x11weight_correction\x18\x02 \x01(\x02H\x00R\x10weightCorrection\x88\x01\x01\x12 \n\x0b\x65nvironment\x18\x03 \x01(\tR\x0b\x65nvironmentB\x14\n\x12_weight_correction\"\x1f\n\x1dSetTokenLeaseConsumedResponse\"H\n\x14ValidateTokenRequest\x12\x30\n\x06tokens\x18\x01 \x03(\x0b\x32\x18.stanza.hub.v1.TokenInfoR\x06tokens\"[\n\tTokenInfo\x12\x1a\n\x05token\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\x05token\x12\x32\n\x05guard\x18\x02 \x01(\x0b\x32\x1c.stanza.hub.v1.GuardSelectorR\x05guard\"k\n\x15ValidateTokenResponse\x12\x14\n\x05valid\x18\x01 \x01(\x08R\x05valid\x12<\n\x0ctokens_valid\x18\x02 \x03(\x0b\x32\x19.stanza.hub.v1.TokenValidR\x0btokensValid\"8\n\nTokenValid\x12\x14\n\x05token\x18\x01 \x01(\tR\x05token\x12\x14\n\x05valid\x18\x02 \x01(\x08R\x05valid*\xac\x01\n\x06Reason\x12\x16\n\x12REASON_UNSPECIFIED\x10\x00\x12\x1b\n\x17REASON_SUFFICIENT_QUOTA\x10\x01\x12\x1d\n\x19REASON_INSUFFICIENT_QUOTA\x10\x02\x12$\n REASON_INSUFFICIENT_QUOTA_PARENT\x10\x03\x12\x10\n\x0cREASON_BURST\x10\x04\x12\x16\n\x12REASON_BEST_EFFORT\x10\x05\x32\x8c\t\n\x0cQuotaService\x12\x94\x02\n\x08GetToken\x12\x1e.stanza.hub.v1.GetTokenRequest\x1a\x1f.stanza.hub.v1.GetTokenResponse\"\xc6\x01\x92\x41\xa8\x01\x12\x10Get Access Token\x1a\x62Get a single token from Stanza Hub for access to given Guard (optional Feature name for priority).J0\n\x03\x32\x30\x30\x12)\n\x02OK\x12#\n!\x1a\x1f.stanza.hub.v1.GetTokenResponse\x82\xd3\xe4\x93\x02\x14\"\x0f/v1/quota/token:\x01*\x12\xb6\x02\n\rGetTokenLease\x12#.stanza.hub.v1.GetTokenLeaseRequest\x1a$.stanza.hub.v1.GetTokenLeaseResponse\"\xd9\x01\x92\x41\xbb\x01\x12\x17Get Access Token Leases\x1aiGet a set of token leases from Stanza Hub for access to given Guard (optional Feature name for priority).J5\n\x03\x32\x30\x30\x12.\n\x02OK\x12(\n&\x1a$.stanza.hub.v1.GetTokenLeaseResponse\x82\xd3\xe4\x93\x02\x14\"\x0f/v1/quota/lease:\x01*\x12\xa7\x02\n\x15SetTokenLeaseConsumed\x12+.stanza.hub.v1.SetTokenLeaseConsumedRequest\x1a,.stanza.hub.v1.SetTokenLeaseConsumedResponse\"\xb2\x01\x92\x41\x91\x01\x12\x15\x43onsume Access Tokens\x1a\x39Inform Stanza Hub that quota access tokens were consumed.J=\n\x03\x32\x30\x30\x12\x36\n\x02OK\x12\x30\n.\x1a,.stanza.hub.v1.SetTokenLeaseConsumedResponse\x82\xd3\xe4\x93\x02\x17\"\x12/v1/quota/consumed:\x01*\x12\x81\x02\n\rValidateToken\x12#.stanza.hub.v1.ValidateTokenRequest\x1a$.stanza.hub.v1.ValidateTokenResponse\"\xa4\x01\x92\x41~\x12\x16Validate Access Tokens\x1a-Validate quota access tokens with Stanza Hub.J5\n\x03\x32\x30\x30\x12.\n\x02OK\x12(\n&\x1a$.stanza.hub.v1.ValidateTokenResponse\x82\xd3\xe4\x93\x02\x1d\"\x18/v1/quota/validatetokens:\x01*B\x8a\x01\n\x11\x63om.stanza.hub.v1B\nQuotaProtoP\x01Z\x13stanza/hub/v1;hubpb\xa2\x02\x03SHX\xaa\x02\rStanza.Hub.V1\xca\x02\rStanza\\Hub\\V1\xe2\x02\x19Stanza\\Hub\\V1\\GPBMetadata\xea\x02\x0fStanza::Hub::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stanza.hub.v1.quota_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\021com.stanza.hub.v1B\nQuotaProtoP\001Z\023stanza/hub/v1;hubpb\242\002\003SHX\252\002\rStanza.Hub.V1\312\002\rStanza\\Hub\\V1\342\002\031Stanza\\Hub\\V1\\GPBMetadata\352\002\017Stanza::Hub::V1'
  _globals['_GETTOKENREQUEST'].fields_by_name['selector']._options = None
  _globals['_GETTOKENREQUEST'].fields_by_name['selector']._serialized_options = b'\342A\001\002'
  _globals['_GETTOKENRESPONSE'].fields_by_name['granted']._options = None
  _globals['_GETTOKENRESPONSE'].fields_by_name['granted']._serialized_options = b'\342A\001\002'
  _globals['_GETTOKENLEASEREQUEST'].fields_by_name['selector']._options = None
  _globals['_GETTOKENLEASEREQUEST'].fields_by_name['selector']._serialized_options = b'\342A\001\002'
  _globals['_GETTOKENLEASERESPONSE'].fields_by_name['granted']._options = None
  _globals['_GETTOKENLEASERESPONSE'].fields_by_name['granted']._serialized_options = b'\342A\001\002'
  _globals['_SETTOKENLEASECONSUMEDREQUEST'].fields_by_name['tokens']._options = None
  _globals['_SETTOKENLEASECONSUMEDREQUEST'].fields_by_name['tokens']._serialized_options = b'\342A\001\002'
  _globals['_TOKENINFO'].fields_by_name['token']._options = None
  _globals['_TOKENINFO'].fields_by_name['token']._serialized_options = b'\342A\001\002'
  _globals['_QUOTASERVICE'].methods_by_name['GetToken']._options = None
  _globals['_QUOTASERVICE'].methods_by_name['GetToken']._serialized_options = b'\222A\250\001\022\020Get Access Token\032bGet a single token from Stanza Hub for access to given Guard (optional Feature name for priority).J0\n\003200\022)\n\002OK\022#\n!\032\037.stanza.hub.v1.GetTokenResponse\202\323\344\223\002\024\"\017/v1/quota/token:\001*'
  _globals['_QUOTASERVICE'].methods_by_name['GetTokenLease']._options = None
  _globals['_QUOTASERVICE'].methods_by_name['GetTokenLease']._serialized_options = b'\222A\273\001\022\027Get Access Token Leases\032iGet a set of token leases from Stanza Hub for access to given Guard (optional Feature name for priority).J5\n\003200\022.\n\002OK\022(\n&\032$.stanza.hub.v1.GetTokenLeaseResponse\202\323\344\223\002\024\"\017/v1/quota/lease:\001*'
  _globals['_QUOTASERVICE'].methods_by_name['SetTokenLeaseConsumed']._options = None
  _globals['_QUOTASERVICE'].methods_by_name['SetTokenLeaseConsumed']._serialized_options = b'\222A\221\001\022\025Consume Access Tokens\0329Inform Stanza Hub that quota access tokens were consumed.J=\n\003200\0226\n\002OK\0220\n.\032,.stanza.hub.v1.SetTokenLeaseConsumedResponse\202\323\344\223\002\027\"\022/v1/quota/consumed:\001*'
  _globals['_QUOTASERVICE'].methods_by_name['ValidateToken']._options = None
  _globals['_QUOTASERVICE'].methods_by_name['ValidateToken']._serialized_options = b'\222A~\022\026Validate Access Tokens\032-Validate quota access tokens with Stanza Hub.J5\n\003200\022.\n\002OK\022(\n&\032$.stanza.hub.v1.ValidateTokenResponse\202\323\344\223\002\035\"\030/v1/quota/validatetokens:\001*'
  _globals['_REASON']._serialized_start=1925
  _globals['_REASON']._serialized_end=2097
  _globals['_GETTOKENREQUEST']._serialized_start=217
  _globals['_GETTOKENREQUEST']._serialized_end=456
  _globals['_GETTOKENRESPONSE']._serialized_start=459
  _globals['_GETTOKENRESPONSE']._serialized_end=664
  _globals['_GETTOKENLEASEREQUEST']._serialized_start=667
  _globals['_GETTOKENLEASEREQUEST']._serialized_end=934
  _globals['_GETTOKENLEASERESPONSE']._serialized_start=936
  _globals['_GETTOKENLEASERESPONSE']._serialized_end=1042
  _globals['_TOKENLEASE']._serialized_start=1045
  _globals['_TOKENLEASE']._serialized_end=1386
  _globals['_SETTOKENLEASECONSUMEDREQUEST']._serialized_start=1389
  _globals['_SETTOKENLEASECONSUMEDREQUEST']._serialized_end=1555
  _globals['_SETTOKENLEASECONSUMEDRESPONSE']._serialized_start=1557
  _globals['_SETTOKENLEASECONSUMEDRESPONSE']._serialized_end=1588
  _globals['_VALIDATETOKENREQUEST']._serialized_start=1590
  _globals['_VALIDATETOKENREQUEST']._serialized_end=1662
  _globals['_TOKENINFO']._serialized_start=1664
  _globals['_TOKENINFO']._serialized_end=1755
  _globals['_VALIDATETOKENRESPONSE']._serialized_start=1757
  _globals['_VALIDATETOKENRESPONSE']._serialized_end=1864
  _globals['_TOKENVALID']._serialized_start=1866
  _globals['_TOKENVALID']._serialized_end=1922
  _globals['_QUOTASERVICE']._serialized_start=2100
  _globals['_QUOTASERVICE']._serialized_end=3264
# @@protoc_insertion_point(module_scope)



from google.api import annotations_pb2 as _annotations_pb2
from google.api import field_behavior_pb2 as _field_behavior_pb2
from stanza.hub.v1 import common_pb2 as _common_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class UpdateStreamsRequest(_message.Message):
    __slots__ = ("guard_name", "environment", "requests", "ended")
    GUARD_NAME_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_FIELD_NUMBER: _ClassVar[int]
    REQUESTS_FIELD_NUMBER: _ClassVar[int]
    ENDED_FIELD_NUMBER: _ClassVar[int]
    guard_name: str
    environment: str
    requests: _containers.RepeatedCompositeFieldContainer[StreamRequest]
    ended: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, guard_name: _Optional[str] = ..., environment: _Optional[str] = ..., requests: _Optional[_Iterable[_Union[StreamRequest, _Mapping]]] = ..., ended: _Optional[_Iterable[str]] = ...) -> None: ...

class UpdateStreamsResponse(_message.Message):
    __slots__ = ("results",)
    RESULTS_FIELD_NUMBER: _ClassVar[int]
    results: _containers.RepeatedCompositeFieldContainer[StreamResult]
    def __init__(self, results: _Optional[_Iterable[_Union[StreamResult, _Mapping]]] = ...) -> None: ...

class StreamRequest(_message.Message):
    __slots__ = ("feature", "tags", "priority_boost", "stream_id", "max_weight", "min_weight")
    FEATURE_FIELD_NUMBER: _ClassVar[int]
    TAGS_FIELD_NUMBER: _ClassVar[int]
    PRIORITY_BOOST_FIELD_NUMBER: _ClassVar[int]
    STREAM_ID_FIELD_NUMBER: _ClassVar[int]
    MAX_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    MIN_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    feature: str
    tags: _containers.RepeatedCompositeFieldContainer[_common_pb2.Tag]
    priority_boost: int
    stream_id: str
    max_weight: float
    min_weight: float
    def __init__(self, feature: _Optional[str] = ..., tags: _Optional[_Iterable[_Union[_common_pb2.Tag, _Mapping]]] = ..., priority_boost: _Optional[int] = ..., stream_id: _Optional[str] = ..., max_weight: _Optional[float] = ..., min_weight: _Optional[float] = ...) -> None: ...

class StreamResult(_message.Message):
    __slots__ = ("stream_id", "allocated_weight")
    STREAM_ID_FIELD_NUMBER: _ClassVar[int]
    ALLOCATED_WEIGHT_FIELD_NUMBER: _ClassVar[int]
    stream_id: str
    allocated_weight: float
    def __init__(self, stream_id: _Optional[str] = ..., allocated_weight: _Optional[float] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stanza/hub/v1/stream.proto
# Protobuf Python Version: 4.25.2
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from stanza.hub.v1 import common_pb2 as stanza_dot_hub_dot_v1_dot_common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1astanza/hub/v1/stream.proto\x12\rstanza.hub.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x1astanza/hub/v1/common.proto\"\xb3\x01\n\x14UpdateStreamsRequest\x12#\n\nguard_name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\tguardName\x12&\n\x0b\x65nvironment\x18\x02 \x01(\tB\x04\xe2\x41\x01\x02R\x0b\x65nvironment\x12\x38\n\x08requests\x18\x03 \x03(\x0b\x32\x1c.stanza.hub.v1.StreamRequestR\x08requests\x12\x14\n\x05\x65nded\x18\x04 \x03(\tR\x05\x65nded\"N\n\x15UpdateStreamsResponse\x12\x35\n\x07results\x18\x01 \x03(\x0b\x32\x1b.stanza.hub.v1.StreamResultR\x07results\"\xfd\x01\n\rStreamRequest\x12\x18\n\x07\x66\x65\x61ture\x18\x01 \x01(\tR\x07\x66\x65\x61ture\x12&\n\x04tags\x18\x02 \x03(\x0b\x32\x12.stanza.hub.v1.TagR\x04tags\x12*\n\x0epriority_boost\x18\x03 \x01(\x05H\x00R\rpriorityBoost\x88\x01\x01\x12!\n\tstream_id\x18\x04 \x01(\tB\x04\xe2\x41\x01\x02R\x08streamId\x12#\n\nmax_weight\x18\x05 \x01(\x02\x42\x04\xe2\x41\x01\x02R\tmaxWeight\x12#\n\nmin_weight\x18\x06 \x01(\x02\x42\x04\xe2\x41\x01\x02R\tminWeightB\x11\n\x0f_priority_boost\"\\\n\x0cStreamResult\x12!\n\tstream_id\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02R\x08streamId\x12)\n\x10\x61llocated_weight\x18\x02 \x01(\x02R\x0f\x61llocatedWeight2\x91\x01\n\x15StreamBalancerService\x12x\n\rUpdateStreams\x12#.stanza.hub.v1.UpdateStreamsRequest\x1a$.stanza.hub.v1.UpdateStreamsResponse\"\x1c\x82\xd3\xe4\x93\x02\x16\"\x11/v1/updatestreams:\x01*B\x8b\x01\n\x11\x63om.stanza.hub.v1B\x0bStreamProtoP\x01Z\x13stanza/hub/v1;hubpb\xa2\x02\x03SHX\xaa\x02\rStanza.Hub.V1\xca\x02\rStanza\\Hub\\V1\xe2\x02\x19Stanza\\Hub\\V1\\GPBMetadata\xea\x02\x0fStanza::Hub::V1b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'stanza.hub.v1.stream_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  _globals['DESCRIPTOR']._options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\021com.stanza.hub.v1B\013StreamProtoP\001Z\023stanza/hub/v1;hubpb\242\002\003SHX\252\002\rStanza.Hub.V1\312\002\rStanza\\Hub\\V1\342\002\031Stanza\\Hub\\V1\\GPBMetadata\352\002\017Stanza::Hub::V1'
  _globals['_UPDATESTREAMSREQUEST'].fields_by_name['guard_name']._options = None
  _globals['_UPDATESTREAMSREQUEST'].fields_by_name['guard_name']._serialized_options = b'\342A\001\002'
  _globals['_UPDATESTREAMSREQUEST'].fields_by_name['environment']._options = None
  _globals['_UPDATESTREAMSREQUEST'].fields_by_name['environment']._serialized_options = b'\342A\001\002'
  _globals['_STREAMREQUEST'].fields_by_name['stream_id']._options = None
  _globals['_STREAMREQUEST'].fields_by_name['stream_id']._serialized_options = b'\342A\001\002'
  _globals['_STREAMREQUEST'].fields_by_name['max_weight']._options = None
  _globals['_STREAMREQUEST'].fields_by_name['max_weight']._serialized_options = b'\342A\001\002'
  _globals['_STREAMREQUEST'].fields_by_name['min_weight']._options = None
  _globals['_STREAMREQUEST'].fields_by_name['min_weight']._serialized_options = b'\342A\001\002'
  _globals['_STREAMRESULT'].fields_by_name['stream_id']._options = None
  _globals['_STREAMRESULT'].fields_by_name['stream_id']._serialized_options = b'\342A\001\002'
  _globals['_STREAMBALANCERSERVICE'].methods_by_name['UpdateStreams']._options = None
  _globals['_STREAMBALANCERSERVICE'].methods_by_name['UpdateStreams']._serialized_options = b'\202\323\344\223\002\026\"\021/v1/updatestreams:\001*'
  _globals['_UPDATESTREAMSREQUEST']._serialized_start=137
  _globals['_UPDATESTREAMSREQUEST']._serialized_end=316
  _globals['_UPDATESTREAMSRESPONSE']._serialized_start=318
  _globals['_UPDATESTREAMSRESPONSE']._serialized_end=396
  _globals['_STREAMREQUEST']._serialized_start=399
  _globals['_STREAMREQUEST']._serialized_end=652
  _globals['_STREAMRESULT']._serialized_start=654
  _globals['_STREAMRESULT']._serialized_end=746
  _globals['_STREAMBALANCERSERVICE']._serialized_start=749
  _globals['_STREAMBALANCERSERVICE']._serialized_end=894
# @@protoc_insertion_point(module_scope)



import asyncio


def noop(*args, **kwargs):
    return None


async def async_noop(*args, **kwargs):
    return None


def async_return(result):
    f = asyncio.Future()
    f.set_result(result)
    return f
